/// MuixApp
///
/// Prototipus d’aplicació mòbil Android/iOS per a la gestió d’assajos, actuacions i catàlegs de figures de colles muixerangueres.
///
/// Grau en Enginyeria Informàtica.
/// Treball Final de Grau. Àrea Desenvolupament multiplataforma d'aplicacions mòbils.
/// Universitat Oberta de Catalunya.
/// Curs 2021-22.
///
/// Estudiant: Hèctor Rubio i Marín (hrubio@uoc.edu)
///
/// MIT License
/// Copyright (c) 2021 Hèctor Rubio i Marín

import 'dart:ui';

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:intl/intl.dart';

class ActivityModel {
  ActivityModel({required this.id, required this.type, required this.title, required this.dateTime, required this.date, required this.time, required this.location, required this.mapLocation});
  final String id;
  final String type;
  final String title;
  final String dateTime;
  final String date;
  final String time;
  final String location;
  final GeoPoint mapLocation;

  factory ActivityModel.fromDocument(DocumentSnapshot doc){
    String docId = "";
    String docType ="";
    String docTitle = "";
    String docDateTime = "";
    String docDate = "";
    String docTime = "";
    String docLocation = "";
    GeoPoint docMapLocation = new GeoPoint(0.0, 0.0);

    try {
      docId = doc.id;
    } catch (e) {
      print("Error (ActivityModel.Id): "+e.toString());
    }
    try {
      docType = doc.get("type");
    } catch (e) {
      print("Error (ActivityModel.Type): "+e.toString());
    }
    try {
      docTitle = doc.get("title");
    } catch (e) {
      print("Error (ActivityModel.Title): "+e.toString());
    }
    try {
      DateTime date = doc.get("datetime").toDate();
      DateFormat format = DateFormat('dd/MM/yyyy - HH:mm');
      docDateTime=format.format(date).toString();
    } catch (e) {
      print("Error (ActivityModel.DateTime): "+e.toString());
    }
    try {
      DateTime date = doc.get("datetime").toDate();
      DateFormat format = DateFormat('dd/MM/yyyy');
      docDate = format.format(date).toString();
    } catch (e) {
      print("Error (ActivityModel.Date): "+e.toString());
    }
    try {
      DateTime date = doc.get("datetime").toDate();
      DateFormat format = DateFormat('HH:mm');
      docTime=format.format(date).toString();
    } catch (e) {
      print("Error (ActivityModel.Time): "+e.toString());
    }
    try {
      docLocation = doc.get("location");
    } catch (e) {
      print("Error (ActivityModel.Location): "+e.toString());
    }
    try {
      docMapLocation = doc.get("maplocation");
    } catch (e) {
      print("Error (ActivityModel.MapLocation): "+e.toString());
    }


    return ActivityModel(
      id: docId,
      type: docType,
      title: docTitle,
      dateTime: docDateTime,
      date: docDate,
      time: docTime,
      location: docLocation,
      mapLocation: docMapLocation,
    );
  }

  Map<String,dynamic> toMap(){
    Map<String,dynamic> data={
      'type': type,
      'title': title,
      'datetime': dateTime,
      'location': location,
      'maplocation': mapLocation,
    };
    return data;
  }

  @override
  int get hashCode => hashValues(id, type, title, dateTime, location, mapLocation);

  @override
  bool operator ==(dynamic other) {
    if (identical(this, other)) return true;
    if (runtimeType != other.runtimeType) return false;
    final ActivityModel otherActivityModel = other;
    return id == otherActivityModel.id &&
        type == otherActivityModel.type &&
        title == otherActivityModel.title &&
        dateTime == otherActivityModel.dateTime &&
        location == otherActivityModel.location &&
        mapLocation == otherActivityModel.mapLocation;
  }

  @override
  String toString() => 'id: $id, type: $type, title: $title, dateTime: $dateTime, location: $location, mapLocation: $mapLocation';
}