/// MuixApp
///
/// Prototipus d’aplicació mòbil Android/iOS per a la gestió d’assajos, actuacions i catàlegs de figures de colles muixerangueres.
///
/// Grau en Enginyeria Informàtica.
/// Treball Final de Grau. Àrea Desenvolupament multiplataforma d'aplicacions mòbils.
/// Universitat Oberta de Catalunya.
/// Curs 2021-22.
///
/// Estudiant: Hèctor Rubio i Marín (hrubio@uoc.edu)
///
/// MIT License
/// Copyright (c) 2021 Hèctor Rubio i Marín

import 'dart:ui';

import 'package:cloud_firestore/cloud_firestore.dart';

class FigureModel {
  FigureModel({required this.id, required this.name, required this.version,
    required this.pictureURL, required this.videoURL, required this.moreInfoURL,
    required this.diagram, required this.description, required this.team,
    required this.positions});

  final String id;
  final String name;
  final String version;
  final String pictureURL;
  final String videoURL;
  final String moreInfoURL;
  final String diagram;
  final String description;
  final String team;
  final int positions;

  Map<String,dynamic> toMap(){
    Map<String,dynamic> data={
      'name': name,
      'version': version,
      'pictureurl': pictureURL,
      'videourl': videoURL,
      'moreinfourl': moreInfoURL,
      'diagram': diagram,
      'description': description,
      'team': team,
      'positions': positions,
    };
    return data;
  }

  factory FigureModel.fromDocument(DocumentSnapshot doc){
    String docId = "";
    String docName ="";
    String docVersion = "";
    String docPictureURL = "";
    String docVideoURL = "";
    String docMoreInfoURL = "";
    String docDiagram = "";
    String docDescription = "";
    String docTeam = "";
    int docPositions = 0;

    try {
      docId = doc.id;
    } catch (e) {
      print("Error (FigureModel.Id): "+e.toString());
    }
    try {
      docName = doc.get("name");
    } catch (e) {
      print("Error (FigureModel.Name): "+e.toString());
    }
    try {
      docVersion = doc.get("version");
    } catch (e) {
      print("Error (FigureModel.Version): "+e.toString());
    }
    try {
      docPictureURL = doc.get("pictureurl");
    } catch (e) {
      print("Error (FigureModel.PictureUrl): "+e.toString());
    }
    try {
      docVideoURL = doc.get("videourl");
    } catch (e) {
      print("Error (FigureModel.VideoUrl): "+e.toString());
    }
    try {
      docMoreInfoURL = doc.get("moreinfourl");
    } catch (e) {
      print("Error (FigureModel.MoreInfoUrl): "+e.toString());
    }
    try {
      docDiagram = doc.get("diagram");
    } catch (e) {
      print("Error (FigureModel.Diagram): "+e.toString());
    }
    try {
      docDescription = doc.get("description");
    } catch (e) {
      print("Error (FigureModel.Description): "+e.toString());
    }
    try {
      docTeam = doc.get("team");
    } catch (e) {
      print("Error (FigureModel.Team): "+e.toString());
    }
    try {
      docPositions = doc.get("positions");
    } catch (e) {
      print("Error (FigureModel.Positions): "+e.toString());
    }

    return FigureModel(
      id: docId,
      name: docName,
      version: docVersion,
      pictureURL: docPictureURL,
      videoURL: docVideoURL,
      moreInfoURL: docMoreInfoURL,
      diagram: docDiagram,
      description: docDescription,
      team: docTeam,
      positions: docPositions
    );
  }

  @override
  int get hashCode => hashValues(id, name, version, pictureURL, videoURL, moreInfoURL, diagram, description, team, positions);

  @override
  bool operator ==(dynamic other) {
    if (identical(this, other)) return true;
    if (runtimeType != other.runtimeType) return false;
    final FigureModel otherFigureModel = other;
    return id == otherFigureModel.id &&
        name == otherFigureModel.name &&
        version == otherFigureModel.version &&
        pictureURL == otherFigureModel.pictureURL &&
        videoURL == otherFigureModel.videoURL &&
        moreInfoURL == otherFigureModel.moreInfoURL &&
        diagram == otherFigureModel.diagram &&
        description == otherFigureModel.description &&
        team == otherFigureModel.team &&
        positions == otherFigureModel.positions;
  }

  @override
  String toString() => 'id: $id, name: $name, version: $version, pictureURL: $pictureURL, videoURL: $videoURL, diagram: $diagram, description: $description, team: $team, positions: $positions';
}