/// MuixApp
///
/// Prototipus d’aplicació mòbil Android/iOS per a la gestió d’assajos, actuacions i catàlegs de figures de colles muixerangueres.
///
/// Grau en Enginyeria Informàtica.
/// Treball Final de Grau. Àrea Desenvolupament multiplataforma d'aplicacions mòbils.
/// Universitat Oberta de Catalunya.
/// Curs 2021-22.
///
/// Estudiant: Hèctor Rubio i Marín (hrubio@uoc.edu)
///
/// MIT License
/// Copyright (c) 2021 Hèctor Rubio i Marín

import 'dart:ui';

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:intl/intl.dart';

class NotificationModel {
  NotificationModel({required this.id, required this.callToAction, required this.title, required this.dateTime, required this.date, required this.time, required this.message, required this.relatedId});
  final String id;
  final bool callToAction;
  final String title;
  final String dateTime;
  final String date;
  final String time;
  final String message;
  final String relatedId;

  factory NotificationModel.fromDocument(DocumentSnapshot doc){
    String docId = "";
    bool docCallToAction =false;
    String docTitle = "";
    String docDateTime = "";
    String docDate = "";
    String docTime = "";
    String docMessage = "";
    String docRelatedId = "";

    try {
      docId = doc.id;
    } catch (e) {
      print("Error (NotificationModel.Id): "+e.toString());
    }
    try {
      docCallToAction = doc.get("calltoaction");
    } catch (e) {
      print("Error (NotificationModel.CallToAction): "+e.toString());
    }
    try {
      docTitle = doc.get("title");
    } catch (e) {
      print("Error (NotificationModel.Title): "+e.toString());
    }
    try {
      DateTime date = doc.get("datetime").toDate();
      DateFormat format = DateFormat('dd/MM/yyyy - HH:mm');
      docDateTime=format.format(date).toString();
    } catch (e) {
      print("Error (NotificationModel.DateTime): "+e.toString());
    }
    try {
      DateTime date = doc.get("datetime").toDate();
      DateFormat format = DateFormat('dd/MM/yyyy');
      docDate = format.format(date).toString();
    } catch (e) {
      print("Error (NotificationModel.Date): "+e.toString());
    }
    try {
      DateTime date = doc.get("datetime").toDate();
      DateFormat format = DateFormat('HH:mm');
      docTime=format.format(date).toString();
    } catch (e) {
      print("Error (NotificationModel.Time): "+e.toString());
    }
    try {
      docMessage = doc.get("message");
    } catch (e) {
      print("Error (NotificationModel.Message): "+e.toString());
    }
    try {
      docRelatedId = doc.get("relatedid");
    } catch (e) {
      print("Error (NotificationModel.RelatedId): "+e.toString());
    }


    return NotificationModel(
      id: docId,
      callToAction: docCallToAction,
      title: docTitle,
      dateTime: docDateTime,
      date: docDate,
      time: docTime,
      message: docMessage,
      relatedId: docRelatedId,
    );
  }

  Map<String,dynamic> toMap(){
    Map<String,dynamic> data={
      'calltoaction': callToAction,
      'title': title,
      'datetime': dateTime,
      'message': message,
      'relatedid': relatedId,
    };
    return data;
  }

  @override
  int get hashCode => hashValues(id, callToAction, title, dateTime, message, relatedId);

  @override
  bool operator ==(dynamic other) {
    if (identical(this, other)) return true;
    if (runtimeType != other.runtimeType) return false;
    final NotificationModel otherNotificationModel = other;
    return id == otherNotificationModel.id &&
        callToAction == otherNotificationModel.callToAction &&
        title == otherNotificationModel.title &&
        dateTime == otherNotificationModel.dateTime &&
        message == otherNotificationModel.message &&
        relatedId == otherNotificationModel.relatedId;
  }

  @override
  String toString() => 'id: $id, callToAction: $callToAction, title: $title, dateTime: $dateTime, message: $message, relatedId: $relatedId';
}