/// MuixApp
///
/// Prototipus d’aplicació mòbil Android/iOS per a la gestió d’assajos, actuacions i catàlegs de figures de colles muixerangueres.
///
/// Grau en Enginyeria Informàtica.
/// Treball Final de Grau. Àrea Desenvolupament multiplataforma d'aplicacions mòbils.
/// Universitat Oberta de Catalunya.
/// Curs 2021-22.
///
/// Estudiant: Hèctor Rubio i Marín (hrubio@uoc.edu)
///
/// MIT License
/// Copyright (c) 2021 Hèctor Rubio i Marín

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:muixapp/src/models/activity_figure_model.dart';
import 'package:muixapp/src/models/attendee_model.dart';
import 'package:muixapp/src/models/figure_model.dart';

class ActivityFiguresProvider {
  final FirebaseFirestore firebaseFirestore;

  ActivityFiguresProvider({required this.firebaseFirestore});

  Future<bool> addActivityFigure(String activityId, FigureModel figureModel) async {
    //print(await getActivityFiguresMaxOrderByActivity(activityId));

    ActivityFigureModel activityFigureModel = ActivityFigureModel(
        id: 'id',
        order: await getActivityFiguresMaxOrderByActivity(activityId)+1,
        name: figureModel.name,
        version: figureModel.version,
        team: figureModel.team,
        activityId: activityId,
        figureId: figureModel.id,
        attendees: Map<String, dynamic>());
    firebaseFirestore.collection('activityfigures').add(activityFigureModel.toMap());


    return true;
  }

  Future<DocumentSnapshot> getActivityFigure(String activityFigureId) async {
    return await firebaseFirestore
        .collection('activityfigures')
        .doc(activityFigureId)
        .get();
  }

  Stream<QuerySnapshot> getActivityFiguresStreamByActivity(String activityId) {
    return firebaseFirestore
        .collection('activityfigures')
        .where("activityid", isEqualTo: activityId)
        .orderBy("order")
        .snapshots();
  }

  Future<QuerySnapshot> getActivityFiguresByActivity(String activityId) async {
    return await firebaseFirestore
        .collection('activityfigures')
        .where("activityid", isEqualTo: activityId)
        .orderBy("order")
        .get();
  }

  Future<int> getActivityFiguresMaxOrderByActivity(String activityId) async {
    int maxValue=0;
     await firebaseFirestore
          .collection('activityfigures')
          .where("activityid", isEqualTo: activityId)
          .orderBy("order", descending: true).limit(1)
          .get()
          .then((QuerySnapshot value) => {
            maxValue=ActivityFigureModel.fromDocument(value.docs[0]).order
           });
     return (maxValue);
  }

  Future<bool> addAttendee(ActivityFigureModel activityFigureModel,
      AttendeeModel attendeeModel, int position) async {
    Map<String, dynamic> activityFigureMap = activityFigureModel.toMap();

    activityFigureMap['attendees'].putIfAbsent(attendeeModel.id,
        () => {'nickname': attendeeModel.nickName, 'position': position});

    firebaseFirestore
        .collection('activityfigures')
        .doc(activityFigureModel.id)
        .update(activityFigureMap);

    return true;
  }

  Future<bool> removeAttendee(ActivityFigureModel activityFigureModel,
      AttendeeModel attendeeModel) async {
    Map<String, dynamic> activityFigureMap = activityFigureModel.toMap();

    activityFigureMap['attendees'].remove(attendeeModel.id);

    firebaseFirestore
        .collection('activityfigures')
        .doc(activityFigureModel.id)
        .update(activityFigureMap);

    return true;
  }
}
