/// MuixApp
///
/// Prototipus d’aplicació mòbil Android/iOS per a la gestió d’assajos, actuacions i catàlegs de figures de colles muixerangueres.
///
/// Grau en Enginyeria Informàtica.
/// Treball Final de Grau. Àrea Desenvolupament multiplataforma d'aplicacions mòbils.
/// Universitat Oberta de Catalunya.
/// Curs 2021-22.
///
/// Estudiant: Hèctor Rubio i Marín (hrubio@uoc.edu)
///
/// MIT License
/// Copyright (c) 2021 Hèctor Rubio i Marín

import 'package:muixapp/src/models/attendee_model.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:muixapp/src/ui/resources/app_constants.dart';
import 'package:muixapp/src/ui/resources/resources.dart';

class AttendeesProvider {
  final FirebaseFirestore firebaseFirestore;

  AttendeesProvider({required this.firebaseFirestore});

  Future<DocumentSnapshot> getAttendee(String attendeeId) async{
    return await firebaseFirestore.collection('attendees').doc(attendeeId).get();
  }

  Stream<QuerySnapshot> getAttendeesStreamByActivity(String activityId) {
    return firebaseFirestore
        .collection('attendees')
        .where("activityid", isEqualTo: activityId)
        .orderBy("value",descending: true)
        .orderBy('nickname')
        .snapshots();
  }
  Stream<QuerySnapshot> getAvailableAttendeesStreamByActivity(String activityId) {
    return firebaseFirestore
        .collection('attendees')
        .where("activityid", isEqualTo: activityId)
        .where("value",isEqualTo: AppConstants.attendingActivity)
        .orderBy('nickname')
        .snapshots();
  }

  Stream<QuerySnapshot> getAttendeeStreamByActivityAndUser(String activityId,String userId) {
    return  firebaseFirestore
        .collection('attendees')
        .where('activityid',isEqualTo: activityId)
        .where('userid',isEqualTo: userId).limit(1).snapshots();
  }
  Future<QuerySnapshot> getAttendeeByActivityAndUser(String activityId,String userId) async{
    return await firebaseFirestore
        .collection('attendees')
        .where('activityid',isEqualTo: activityId)
        .where('userid',isEqualTo: userId).limit(1).get();
  }

  void notAttending(String activityId, String userId, String nickName) async {
    chooseAttendance(activityId, userId, AppConstants.notAttendingActivity, nickName);
  }
  void maybeAttending(String activityId, String userId, String nickname) {
    chooseAttendance(activityId, userId, AppConstants.maybeAttendingActivity, nickname);
  }
  void attending(String activityId, String userId, String nickName) {
    chooseAttendance(activityId, userId, AppConstants.attendingActivity, nickName);
  }

  void chooseAttendance(String activityId, String userId, int value, String nickname) {
    Future<QuerySnapshot> snapshot = getAttendeeByActivityAndUser(activityId, userId);
    snapshot.then((querySnapshot) =>{
      (querySnapshot.size>0? updateAttendeeAnswer(AttendeeModel.fromDocument(querySnapshot.docs[0]),value) : addAttendee(activityId, userId, nickname, value, 0))
    });
  }
  Future<bool> updateAttendeeAnswer(AttendeeModel attendee, int value) async {
    Map<String,dynamic> attendeeMap = attendee.toMap();
    attendeeMap['value']=value;

    // needed to reset this in case attendee was already taking part of different activity figures

    if(value==AppConstants.maybeAttendingActivity || value==AppConstants.notAttendingActivity) {
      attendeeMap['usedin'] = 0;
    }
    firebaseFirestore
        .collection('attendees')
        .doc(attendee.id)
        .update(attendeeMap);
    return true;
  }

  Future<bool> addAttendee(String activityId, String userId, String nickName, int value, int usedIn) async {
    AttendeeModel attendeeModel = AttendeeModel(id: '', userId: userId, activityId: activityId, value: value, nickName: nickName, usedIn: usedIn);
    firebaseFirestore
        .collection('attendees').add(attendeeModel.toMap());
    return true;
  }

  Future<bool> updateAttendee(AttendeeModel attendee) async {
    Map<String,dynamic> attendeeMap = attendee.toMap();
    firebaseFirestore
        .collection('attendees')
        .doc(attendee.id)
        .update(attendeeMap);
    return true;
  }

  Future<bool> increaseAttendeeUsedIn(AttendeeModel attendee) async {
    Map<String,dynamic> attendeeMap = attendee.toMap();
    attendeeMap['usedin']+=1;
    firebaseFirestore
        .collection('attendees')
        .doc(attendee.id)
        .update(attendeeMap);
    return true;
  }

  Future<bool> decreaseAttendeeUsedIn(AttendeeModel attendee) async {
    Map<String,dynamic> attendeeMap = attendee.toMap();
    attendeeMap['usedin']-=1;
    firebaseFirestore
        .collection('attendees')
        .doc(attendee.id)
        .update(attendeeMap);
    return true;
  }

}
