/// MuixApp
///
/// Prototipus d’aplicació mòbil Android/iOS per a la gestió d’assajos, actuacions i catàlegs de figures de colles muixerangueres.
///
/// Grau en Enginyeria Informàtica.
/// Treball Final de Grau. Àrea Desenvolupament multiplataforma d'aplicacions mòbils.
/// Universitat Oberta de Catalunya.
/// Curs 2021-22.
///
/// Estudiant: Hèctor Rubio i Marín (hrubio@uoc.edu)
///
/// MIT License
/// Copyright (c) 2021 Hèctor Rubio i Marín

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/material.dart';
import 'package:muixapp/src/ui/resources/resources.dart';
import 'package:shared_preferences/shared_preferences.dart';

class AuthenticationProvider extends ChangeNotifier {
  final FirebaseFirestore firebaseFirestore;
  final FirebaseAuth firebaseAuth;
  final SharedPreferences preferences;

  AuthenticationProvider({
    required this.firebaseFirestore,
    required this.firebaseAuth,
    required this.preferences,
  });

  bool isLoggedIn(){
    return preferences.getString('uid')!=null;
  }

  bool isManager(){
    return preferences.getBool('ismanager')!;
  }

  bool isTrainer(){
    return preferences.getBool('istrainer')!;
  }

  Future<bool> login(String email, String password, BuildContext context) async{
    try {
      User? user = (await firebaseAuth.signInWithEmailAndPassword(
          email: email, password: password)).user;
      if (user != null) {
        // login ok

        // set user details as shared preferences

        await preferences.setString('uid', user.uid);
        await preferences.setString('useremail', user.email!);
        await preferences.setBool('ismanager', false);
        await preferences.setBool('istrainer', false);

        return true;
      }
      else {
        // login error
        ScaffoldMessenger.of(context).showSnackBar(
          SnackBar(
            content: Container(child:Text(AppTexts.authenticationGenericError)),
            padding: EdgeInsets.all(20),
          ),
        );
        return false;
      }
    } on FirebaseAuthException catch (error) {
      ScaffoldMessenger.of(context).showSnackBar(
          SnackBar(
            content: Container(child:Text(getAuthenticationErrorMessage(error.code, error.toString()))),
            padding: EdgeInsets.all(20),
          ),
      );
      return false;
    }
  }

  Future<void> logout() async {
    await firebaseAuth.signOut();
    await preferences.remove('uid');
    await preferences.remove('useremail');
    await preferences.remove('ismanager');
    await preferences.remove('istrainer');
  }

  //String firstName, String lastName, String nickName, String mobilePhone,

  Future<bool> register(String email, String password, BuildContext context) async{
    try {
      User? user = (await firebaseAuth.createUserWithEmailAndPassword(email: email, password: password)).user;
      if (user != null) {

        // user register ok

        // set user details as shared preferences

        await preferences.setString('uid', user.uid);
        await preferences.setString('useremail', user.email!);
        await preferences.setBool('ismanager', false);
        await preferences.setBool('istrainer', false);

        return true;
      }
      else {
        // register error
        ScaffoldMessenger.of(context).showSnackBar(
          SnackBar(
            content: Container(child:Text(AppTexts.authenticationGenericError)),
            padding: EdgeInsets.all(20),
          ),
        );
        return false;
      }
    } on FirebaseAuthException catch (error) {
      ScaffoldMessenger.of(context).showSnackBar(
        SnackBar(
          content: Container(child:Text(getAuthenticationErrorMessage(error.code, error.toString()))),
          padding: EdgeInsets.all(20),
        ),
      );
      return false;
    }
  }

  String getAuthenticationErrorMessage(String code, String message) {
    String errorMessage=message;

    print(code);

    switch(code){
      case 'email-already-in-use':
        errorMessage=AppTexts.authenticationEmailAlreadyInUse;
        break;
      case 'missing-email':
        errorMessage=AppTexts.authenticationMissingEmail;
        break;
      case 'invalid-email':
        errorMessage=AppTexts.authenticationInvalidEmail;
        break;
      case 'user-not-found':
        errorMessage=AppTexts.authenticationUserNotFound;
        break;
      case 'weak-password':
        errorMessage=AppTexts.authenticationWeakPassword;
        break;
      case 'wrong-password':
        errorMessage=AppTexts.authenticationWrongPassword;
        break;
    }

    return errorMessage;
  }
}