/// MuixApp
///
/// Prototipus d’aplicació mòbil Android/iOS per a la gestió d’assajos, actuacions i catàlegs de figures de colles muixerangueres.
///
/// Grau en Enginyeria Informàtica.
/// Treball Final de Grau. Àrea Desenvolupament multiplataforma d'aplicacions mòbils.
/// Universitat Oberta de Catalunya.
/// Curs 2021-22.
///
/// Estudiant: Hèctor Rubio i Marín (hrubio@uoc.edu)
///
/// MIT License
/// Copyright (c) 2021 Hèctor Rubio i Marín

import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/material.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:muixapp/src/providers/activity_figures_provider.dart';
import 'package:muixapp/src/ui/pages/authentication/login.dart';
import 'package:provider/provider.dart';

import 'package:muixapp/src/providers/providers.dart';
import 'package:muixapp/src/ui/resources/app_texts.dart';
import 'package:shared_preferences/shared_preferences.dart';

class App extends StatelessWidget {

  final FirebaseFirestore firebaseFirestore = FirebaseFirestore.instance;
  final SharedPreferences preferences;

  App({required this.preferences});

  @override
  Widget build(BuildContext context) {

    return MultiProvider(
        providers: [
          ChangeNotifierProvider<AuthenticationProvider>(
            create: (_) => AuthenticationProvider(
              firebaseAuth: FirebaseAuth.instance,
              firebaseFirestore: this.firebaseFirestore,
              preferences: this.preferences,
            ),
          ),
          Provider<ActivitiesProvider>(
            create: (_) => ActivitiesProvider(
              firebaseFirestore: this.firebaseFirestore,
            )
          ),
          Provider<ActivityFiguresProvider>(
              create: (_) => ActivityFiguresProvider(
                firebaseFirestore: this.firebaseFirestore,
              )
          ),
          Provider<AttendeesProvider>(
              create: (_) => AttendeesProvider(
                firebaseFirestore: this.firebaseFirestore,
              )
          ),
          Provider<FiguresProvider>(
              create: (_) => FiguresProvider(
                firebaseFirestore: this.firebaseFirestore,
              )
          ),
          Provider<NotificationsProvider>(
              create: (_) => NotificationsProvider(
                firebaseFirestore: this.firebaseFirestore,
              )
          ),
          Provider<PreferencesProvider>(
              create: (_) => PreferencesProvider(
                  preferences: this.preferences,
              ),
          ),
          Provider<ProfilesProvider>(
              create: (_) => ProfilesProvider(
                firebaseFirestore: this.firebaseFirestore,
              )
          ),
        ],
        child: MaterialApp(
            title: AppTexts.appTitle,
            theme: ThemeData.light(),
            home: Login(),
            debugShowCheckedModeBanner: false,
          ),
        );
  }
}

