/// MuixApp
///
/// Prototipus d’aplicació mòbil Android/iOS per a la gestió d’assajos, actuacions i catàlegs de figures de colles muixerangueres.
///
/// Grau en Enginyeria Informàtica.
/// Treball Final de Grau. Àrea Desenvolupament multiplataforma d'aplicacions mòbils.
/// Universitat Oberta de Catalunya.
/// Curs 2021-22.
///
/// Estudiant: Hèctor Rubio i Marín (hrubio@uoc.edu)
///
/// MIT License
/// Copyright (c) 2021 Hèctor Rubio i Marín

import 'package:flutter/material.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:material_design_icons_flutter/material_design_icons_flutter.dart';
import 'package:muixapp/src/ui/widgets/authentication/no_access_no_logged_in_user.dart';
import 'package:provider/provider.dart';

import 'package:muixapp/src/models/activity_model.dart';
import 'package:muixapp/src/providers/providers.dart';
import 'package:muixapp/src/ui/widgets/activities/activities_list_tile.dart';

class ActivitiesList extends StatefulWidget {
  ActivitiesList({Key? key}) : super(key: key);

  @override
  State createState() => ActivitiesListState();
}

class ActivitiesListState extends State<ActivitiesList> {
  ActivitiesListState({Key? key});

  TextEditingController searchController = TextEditingController();

  late ActivitiesProvider activitiesProvider;
  late AuthenticationProvider authenticationProvider;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    authenticationProvider = Provider.of<AuthenticationProvider>(context);
    activitiesProvider = context.read<ActivitiesProvider>();
  }

  @override
  Widget build(BuildContext context) {
    if (authenticationProvider.isLoggedIn())
      return Container(child:Column(
      children:[
        SizedBox(height: 10),
        Container(
          padding: EdgeInsets.symmetric(horizontal: 8.0), //.all(4.0),
          child: TextField(
            onChanged: (value) {
              setState(() {});
            },
            controller: searchController,
            decoration: InputDecoration(
                hintText: 'Cerca',
                prefixIcon: Icon(MdiIcons.magnify),
                border: OutlineInputBorder(
                    borderRadius: BorderRadius.all(Radius.circular(5.0))
                )
            ),
            ),
        ),
        SizedBox(height: 4),
        Flexible(
            fit: FlexFit.tight,
            child:Container(
          child: StreamBuilder<QuerySnapshot>(
            stream: activitiesProvider.getActivitiesStream(),
            builder: (context, AsyncSnapshot<QuerySnapshot> snapshot) {
              if (snapshot.hasData) {
                int counter = (snapshot.data?.docs.length ?? 0);
                if (counter > 0) // return Text(counter.toString());
                  return ListView.builder(
                    padding: EdgeInsets.symmetric(horizontal: 8),
                    itemBuilder: (context, index) =>
                        buildItem(
                            context,
                            snapshot.data?.docs[index],
                            (searchController.text.isEmpty? '': searchController.text.toLowerCase()),
                        ),
                    itemCount: snapshot.data?.docs.length,
                  );
              } else if (snapshot.hasError) {
                return Text(snapshot.error.toString());
              }
              return Center(child: CircularProgressIndicator());
            },
          ),
        )
        )
      ]
    ),);
    else
      return NoAccessNoLoggedInUser();
  }

  Widget buildItem(BuildContext context, DocumentSnapshot? document, String searchFilter) {
    if (document != null) {
      ActivityModel activity = ActivityModel.fromDocument(document);
      if (activity.title.toLowerCase().contains(searchFilter))
        return Container(
        padding: EdgeInsets.symmetric(vertical: 5.0), //.all(4.0),
        child: ActivitiesListTile(activity, context),
      );
      else
        return SizedBox.shrink();
    } else {
      return SizedBox.shrink();
    }
  }
}
