/// MuixApp
///
/// Prototipus d’aplicació mòbil Android/iOS per a la gestió d’assajos, actuacions i catàlegs de figures de colles muixerangueres.
///
/// Grau en Enginyeria Informàtica.
/// Treball Final de Grau. Àrea Desenvolupament multiplataforma d'aplicacions mòbils.
/// Universitat Oberta de Catalunya.
/// Curs 2021-22.
///
/// Estudiant: Hèctor Rubio i Marín (hrubio@uoc.edu)
///
/// MIT License
/// Copyright (c) 2021 Hèctor Rubio i Marín

import 'package:flutter/material.dart';
import 'package:muixapp/src/models/activity_model.dart';
import 'package:muixapp/src/providers/providers.dart';
import 'package:muixapp/src/ui/resources/resources.dart';
import 'package:muixapp/src/ui/widgets/activities/activity_type_icon.dart';
import 'package:muixapp/src/ui/widgets/activities/activity_type_text.dart';
import 'package:muixapp/src/ui/widgets/activities/options_action.dart';
import 'package:muixapp/src/ui/widgets/activities/user_attendance.dart';
import 'package:muixapp/src/ui/widgets/common/app_page_bar.dart';
import 'package:google_maps_flutter/google_maps_flutter.dart';
import 'package:provider/provider.dart';

class Activity extends StatefulWidget {
  final ActivityModel activity;
  final String appMainBarBackText;

  const Activity({Key? key, required this.activity, required this.appMainBarBackText}) : super(key: key);

  @override
  _ActivityState createState() => _ActivityState(activity: this.activity, appMainBarBackText: this.appMainBarBackText);
}

class _ActivityState extends State<Activity> {
  _ActivityState({required this.activity,required this.appMainBarBackText});

  ActivityModel activity;
  String appMainBarBackText;
  late GoogleMapController mapController;
  late PreferencesProvider preferencesProvider;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    preferencesProvider = Provider.of<PreferencesProvider>(context);
  }

  void _onMapCreated(GoogleMapController controller) {
    mapController = controller;
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      child: Scaffold(
        appBar: AppPageBar(
          leadingText: this.appMainBarBackText,
          barActions: [OptionsAction(activityModel: this.activity)],
        ),
        body: Container(
          margin: EdgeInsets.symmetric(horizontal: 5.0),
          child: Column(
            children: [
              SizedBox(
                height: 10.0,
              ),

              // Main data row

              Row(
                children: [
                  // Left column - Icon
                  ActivityTypeIcon(activity.type),
                  // Central column - Activity info
                  Flexible(
                    fit: FlexFit.tight,
                    child: Container(
                      margin: EdgeInsets.symmetric(vertical: 5.0),
                      height: 80,
                      child: Column(
                        mainAxisAlignment: MainAxisAlignment.start,
                        children: [
                          SizedBox(
                            height: 4.0,
                          ),
                          Row(
                            mainAxisAlignment: MainAxisAlignment.spaceBetween,
                            children: [
                              ActivityTypeText(activity.type),
                              Text(
                                activity.dateTime + " h",
                                style: AppStyles.activityListTileDateTimeStyle,
                              ),
                            ],
                          ),
                          SizedBox(height: 8),
                          Row(
                            crossAxisAlignment: CrossAxisAlignment.start,
                            children: [
                              Expanded(
                                child: Column(
                                  crossAxisAlignment: CrossAxisAlignment.start,
                                  children: [
                                    Text(
                                      activity.title,
                                      overflow: TextOverflow.ellipsis,
                                      maxLines: 3,
                                      softWrap: true,
                                    ),
                                  ],
                                ),
                              ),
                            ],
                          ),
                        ],
                      ),
                    ),
                  ),
                  SizedBox(width: 20),
                ],
              ),
              SizedBox(
                height: 10.0,
              ),

              // User attendance row

              Container(
                  margin: EdgeInsets.symmetric(horizontal: 10.0),
                  child: UserAttendance(activityId:activity.id,userId: preferencesProvider.getKeyStringValue('uid')!, nickName: preferencesProvider.getKeyStringValue('usernickname')!,), // RSVP row
              ),
              SizedBox(
                height: 15.0,
              ),

              // Activity Location row

              Container(
                  margin: EdgeInsets.symmetric(horizontal: 10.0),
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      AppIcons.activityMap,
                      Flexible(
                        fit: FlexFit.tight,
                        child: Container(
                          margin: EdgeInsets.symmetric(horizontal: 10.0),
                          height: 60,
                          child: Column(
                            mainAxisAlignment: MainAxisAlignment.start,
                            crossAxisAlignment: CrossAxisAlignment.start,
                            children: [
                              Text(
                                activity.location,
                                overflow: TextOverflow.ellipsis,
                                maxLines: 3,
                                softWrap: true,
                              ),
                            ],
                          ),
                        ),
                      ),
                    ],
                  )
              ),

              // Google Maps row

              Flexible(
                fit: FlexFit.tight,
                child: GoogleMap(
                    mapType: MapType.hybrid,
                    markers: {
                      Marker(
                        markerId: MarkerId('Ubicació'),
                        position: LatLng(activity.mapLocation.latitude,activity.mapLocation.longitude),
                      ),
                    },
                    onMapCreated: _onMapCreated,
                    initialCameraPosition: CameraPosition(
                      target: LatLng(activity.mapLocation.latitude,activity.mapLocation.longitude),
                      zoom: 16.0,
                    ),
                  ),
              ),
            ],
          ),
        ),
      ),
    );
  }
}
