/// MuixApp
///
/// Prototipus d’aplicació mòbil Android/iOS per a la gestió d’assajos, actuacions i catàlegs de figures de colles muixerangueres.
///
/// Grau en Enginyeria Informàtica.
/// Treball Final de Grau. Àrea Desenvolupament multiplataforma d'aplicacions mòbils.
/// Universitat Oberta de Catalunya.
/// Curs 2021-22.
///
/// Estudiant: Hèctor Rubio i Marín (hrubio@uoc.edu)
///
/// MIT License
/// Copyright (c) 2021 Hèctor Rubio i Marín

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';
import 'package:muixapp/src/models/activity_model.dart';
import 'package:muixapp/src/models/attendee_model.dart';
import 'package:muixapp/src/providers/providers.dart';
import 'package:muixapp/src/ui/resources/resources.dart';
import 'package:muixapp/src/ui/widgets/activities/activity_info_bar.dart';
import 'package:muixapp/src/ui/widgets/authentication/no_access_no_logged_in_user.dart';
import 'package:muixapp/src/ui/widgets/common/app_page_bar.dart';
import 'package:provider/provider.dart';

class ActivityAvailablePeopleList extends StatefulWidget {
  final ActivityModel activityModel;

  const ActivityAvailablePeopleList({required this.activityModel, Key? key})
      : super(key: key);

  @override
  _ActivityAvailablePeopleListState createState() =>
      _ActivityAvailablePeopleListState(this.activityModel);
}

class _ActivityAvailablePeopleListState extends State<ActivityAvailablePeopleList> {
  final ActivityModel activityModel;

  _ActivityAvailablePeopleListState(this.activityModel);

  late AttendeesProvider attendeesProvider;
  late AuthenticationProvider authenticationProvider;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    authenticationProvider = Provider.of<AuthenticationProvider>(context);
    attendeesProvider = context.read<AttendeesProvider>();
  }

  @override
  Widget build(BuildContext context) {
    if (authenticationProvider.isLoggedIn())
      return Container(
        child: Scaffold(
            appBar: AppPageBar(
              leadingText: AppTexts.activityAvailablePeopleBackMenuItem,
              barActions: [],
            ),
            body: Container(
              child: Column(children: [
                ActivityInfoBar(activityModel: activityModel),
                SizedBox(height: 10),
                Flexible(
                  fit: FlexFit.tight,
                  child: StreamBuilder<QuerySnapshot>(
                    stream: attendeesProvider
                        .getAvailableAttendeesStreamByActivity(this.activityModel.id),
                    builder: (context, AsyncSnapshot<QuerySnapshot> snapshot) {
                      if (snapshot.hasData) {
                        int counter = (snapshot.data?.docs.length ?? 0);
                        if (counter > 0) {
                          // return Text(counter.toString());
                          return ListView.builder(
                            padding: EdgeInsets.symmetric(horizontal: 8),
                            itemBuilder: (context, index) => buildItem(
                              context,
                              snapshot.data?.docs[index],
                            ),
                            itemCount: snapshot.data?.docs.length,
                          );
                        } else
                          return Center(
                              child: Container(
                                margin: EdgeInsets.all(20),
                                  child:Text(AppTexts.activityAvailablePeopleNoPeopleMenuItem))
                          );
                      } else if (snapshot.hasError) {
                        return Text(snapshot.error.toString());
                      }
                      return Center(child: CircularProgressIndicator());
                    },
                  ),
                ),
                SizedBox(height: 10),
              ]),
            )),
      );
    else
      return NoAccessNoLoggedInUser();
  }

  Widget buildItem(BuildContext context, DocumentSnapshot? document) {
    if (document != null) {
      AttendeeModel attendeeModel = AttendeeModel.fromDocument(document);
      return Container(
          padding: EdgeInsets.all(10),
          height: 70,
          decoration: BoxDecoration(
            border: Border.all(width: 1, color: AppColours.midGray),
          ),
          child: Row(
            children: [
              AppIcons.standardUserAttendance,
              Flexible(
                fit: FlexFit.tight,
                child: Text(
                  attendeeModel.nickName,
                  style: AppStyles.activityAttendanceTextStyle,
                ),
              ),
            ],
          ));
    } else {
      return SizedBox.shrink();
    }
  }
}
