/// MuixApp
///
/// Prototipus d’aplicació mòbil Android/iOS per a la gestió d’assajos, actuacions i catàlegs de figures de colles muixerangueres.
///
/// Grau en Enginyeria Informàtica.
/// Treball Final de Grau. Àrea Desenvolupament multiplataforma d'aplicacions mòbils.
/// Universitat Oberta de Catalunya.
/// Curs 2021-22.
///
/// Estudiant: Hèctor Rubio i Marín (hrubio@uoc.edu)
///
/// MIT License
/// Copyright (c) 2021 Hèctor Rubio i Marín

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';
import 'package:muixapp/src/models/activity_figure_model.dart';
import 'package:muixapp/src/models/activity_model.dart';
import 'package:muixapp/src/models/attendee_model.dart';
import 'package:muixapp/src/providers/providers.dart';
import 'package:muixapp/src/ui/resources/resources.dart';
import 'package:muixapp/src/ui/widgets/activities/activity_figure_info_bar.dart';
import 'package:muixapp/src/ui/widgets/authentication/no_access_no_logged_in_user.dart';
import 'package:muixapp/src/ui/widgets/common/app_page_bar.dart';
import 'package:provider/provider.dart';

class ActivityFigureAddPeoplePosition extends StatefulWidget {
  final ActivityModel activityModel;
  final ActivityFigureModel activityFigureModel;
  final AttendeeModel attendeeModel;

  const ActivityFigureAddPeoplePosition(
      {required this.activityModel,
      required this.activityFigureModel,
      required this.attendeeModel,
      Key? key})
      : super(key: key);

  @override
  _ActivityFigureAddPeoplePositionState createState() =>
      _ActivityFigureAddPeoplePositionState(
          this.activityModel, this.activityFigureModel, this.attendeeModel);
}

class _ActivityFigureAddPeoplePositionState
    extends State<ActivityFigureAddPeoplePosition> {
  final ActivityModel activityModel;
  final ActivityFigureModel activityFigureModel;
  final AttendeeModel attendeeModel;
  final positionTextEditingController = new TextEditingController();

  _ActivityFigureAddPeoplePositionState(
      this.activityModel, this.activityFigureModel, this.attendeeModel);

  late AttendeesProvider attendeesProvider;
  late ActivityFiguresProvider activityFiguresProvider;
  late AuthenticationProvider authenticationProvider;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    authenticationProvider = Provider.of<AuthenticationProvider>(context);
    attendeesProvider = context.read<AttendeesProvider>();
    activityFiguresProvider = context.read<ActivityFiguresProvider>();
  }

  @override
  void dispose() {
    positionTextEditingController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (authenticationProvider.isLoggedIn())
      return new GestureDetector(
          onTap: () => FocusManager.instance.primaryFocus?.unfocus(),
          child: Scaffold(
              resizeToAvoidBottomInset: false,
              backgroundColor: AppColours.white,
              body: SafeArea(
                  child: Container(
                child: Scaffold(
                    appBar: AppPageBar(
                      leadingText: AppTexts.backAppMainBarNavigation,
                      barActions: [],
                    ),
                    body: Container(
                      child: Column(children: [
                        ActivityFigureInfoBar(activityModel: activityModel, activityFigureModel: activityFigureModel),
                        SizedBox(height: 10),
                        Flexible(
                            fit: FlexFit.tight,
                            child: Container(
                              margin: EdgeInsets.all(10),
                              padding: EdgeInsets.all(10),
                              decoration: BoxDecoration(
                                border: Border.all(
                                    width: 1, color: AppColours.midGray),
                              ),
                              child: Row(
                                mainAxisAlignment: MainAxisAlignment.start,
                                crossAxisAlignment: CrossAxisAlignment.start,
                                children: [
                                  AppIcons.standardUserAttendance,
                                  Flexible(
                                    fit: FlexFit.tight,
                                    child: Column(
                                        mainAxisAlignment:
                                            MainAxisAlignment.start,
                                        crossAxisAlignment:
                                            CrossAxisAlignment.start,
                                        children: [
                                          Text(
                                            attendeeModel.nickName,
                                            style: AppStyles
                                                .activityFiguresAssignPeopleNickNameTimeStyle,
                                          ),
                                          SizedBox(
                                            height: 15,
                                          ),
                                          Text(AppTexts
                                              .activityFigureAddPeoplePositionText),
                                          SizedBox(
                                            height: 5,
                                          ),
                                          Row(
                                              mainAxisAlignment:
                                                  MainAxisAlignment.start,
                                              crossAxisAlignment:
                                                  CrossAxisAlignment.start,
                                              children: [
                                                Container(
                                                    margin: EdgeInsets.fromLTRB(
                                                        0, 10, 0, 0),
                                                    width: 220,
                                                    child: TextField(
                                                      controller:
                                                          positionTextEditingController,
                                                      keyboardType: TextInputType.number,
                                                      style: TextStyle(
                                                        fontSize: 14,
                                                        height: 1,
                                                      ),
                                                      decoration:
                                                          InputDecoration(
                                                              filled: true,
                                                              fillColor:
                                                                  AppColours
                                                                      .white,
                                                              hintText: AppTexts
                                                                  .activityFigureAddPeoplePositionHint,
                                                              isDense: true,
                                                              contentPadding:
                                                                  EdgeInsets
                                                                      .symmetric(
                                                                vertical: 8,
                                                                horizontal: 8,
                                                              ),
                                                              border:
                                                                  OutlineInputBorder(
                                                                borderRadius:
                                                                    BorderRadius
                                                                        .all(Radius
                                                                            .zero),
                                                              )),
                                                      enableSuggestions: false,
                                                      autocorrect: false,
                                                    )),
                                                IconButton(
                                                    onPressed: () async{
                                                      if (validateForm())
                                                        if (await addPerson())
                                                          Navigator.popUntil(
                                                              context,
                                                              (route) => route.settings.name == "assignPeople"
                                                          );
                                                    },
                                                    icon: AppIcons.checkIcon),
                                              ]),
                                        ]),
                                  ),
                                ],
                              ),
                            )
                            ),
                        SizedBox(height: 10),
                      ]),
                    )),
              ))));
    else
      return NoAccessNoLoggedInUser();
  }

  Widget buildItem(BuildContext context, ActivityModel activityModel,
      ActivityFigureModel activityFigureModel, DocumentSnapshot? document) {
    if (document != null) {
      AttendeeModel attendeeModel = AttendeeModel.fromDocument(document);
      return Container(
          padding: EdgeInsets.fromLTRB(10, 10, 0, 10),
          height: 70,
          decoration: BoxDecoration(
            border: Border.all(width: 1, color: AppColours.midGray),
          ),
          child: Row(
            children: [
              AppIcons.standardUserAttendance,
              Flexible(
                fit: FlexFit.tight,
                child: Column(
                    mainAxisAlignment: MainAxisAlignment.start,
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Text(
                        attendeeModel.nickName,
                        style: AppStyles
                            .activityFiguresAssignPeopleNickNameTimeStyle,
                      ),
                      SizedBox(
                        height: 5,
                      ),
                      Text(
                        (attendeeModel.usedIn == 1
                            ? attendeeModel.usedIn.toString() + ' figura'
                            : attendeeModel.usedIn.toString() + ' figures'),
                        style: AppStyles
                            .activityFiguresAssignPeopleUsedInTimeStyle,
                      ),
                    ]),
              ),
              IconButton(
                  onPressed: () {},
                  icon: AppIcons.activityFigureAssignPeopleAdd),
            ],
          ));
    } else {
      return SizedBox.shrink();
    }
  }

  bool validateForm() {
    //TODO Improve validation of position (check whether it is possible/busy or not, etc)
    String errorMessage = '';

    if (positionTextEditingController.text.isEmpty){
      errorMessage = AppTexts.activityFigureAddPeoplePositiveNumberExpected;
    } else if (int.tryParse(positionTextEditingController.text)==null)
      errorMessage = AppTexts.activityFigureAddPeoplePositiveNumberExpected;

    if (errorMessage.isEmpty)
      return true;
    else {
      ScaffoldMessenger.of(context).showSnackBar(
        SnackBar(
          content: Container(child:Text(errorMessage)),
          padding: EdgeInsets.all(20),
        ),
      );
      return false;
    }
  }

  Future<bool> addPerson() async {
    if (await attendeesProvider
        .increaseAttendeeUsedIn(
    attendeeModel)){
      if (await activityFiguresProvider.addAttendee(activityFigureModel, attendeeModel, int.parse(positionTextEditingController.text)))
        return true;
      else
        return false;
    }
    else return false;
  }
}
