/// MuixApp
///
/// Prototipus d’aplicació mòbil Android/iOS per a la gestió d’assajos, actuacions i catàlegs de figures de colles muixerangueres.
///
/// Grau en Enginyeria Informàtica.
/// Treball Final de Grau. Àrea Desenvolupament multiplataforma d'aplicacions mòbils.
/// Universitat Oberta de Catalunya.
/// Curs 2021-22.
///
/// Estudiant: Hèctor Rubio i Marín (hrubio@uoc.edu)
///
/// MIT License
/// Copyright (c) 2021 Hèctor Rubio i Marín

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';
import 'package:muixapp/src/models/activity_figure_model.dart';
import 'package:muixapp/src/models/activity_model.dart';
import 'package:muixapp/src/models/attendee_model.dart';
import 'package:muixapp/src/models/figure_model.dart';
import 'package:muixapp/src/providers/providers.dart';
import 'package:muixapp/src/ui/pages/activities/activity_figure_add_people.dart';
import 'package:muixapp/src/ui/pages/activities/activity_figure_remove_people.dart';
import 'package:muixapp/src/ui/resources/resources.dart';
import 'package:muixapp/src/ui/widgets/activities/activity_figure_diagram.dart';
import 'package:muixapp/src/ui/widgets/activities/activity_figure_info_bar.dart';
import 'package:muixapp/src/ui/widgets/authentication/no_access_no_logged_in_user.dart';
import 'package:muixapp/src/ui/widgets/common/app_page_bar.dart';
import 'package:provider/provider.dart';

class ActivityFigureAssignPeople extends StatefulWidget {
  final ActivityModel activityModel;
  final ActivityFigureModel activityFigureModel;

  const ActivityFigureAssignPeople({required this.activityModel, required this.activityFigureModel, Key? key})
      : super(key: key);

  @override
  _ActivityFigureAssignPeopleState createState() =>
      _ActivityFigureAssignPeopleState(this.activityModel, this.activityFigureModel);
}

class _ActivityFigureAssignPeopleState extends State<ActivityFigureAssignPeople> {
  final ActivityModel activityModel;
  final ActivityFigureModel activityFigureModel;

  _ActivityFigureAssignPeopleState(this.activityModel, this.activityFigureModel);

  late FiguresProvider figuresProvider;
  late AuthenticationProvider authenticationProvider;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    authenticationProvider = Provider.of<AuthenticationProvider>(context);
    figuresProvider = context.read<FiguresProvider>();
  }

  @override
  Widget build(BuildContext context) {
    if (authenticationProvider.isLoggedIn()) {

      return FutureBuilder(
          future: figuresProvider.getFigure(activityFigureModel.figureId),
          builder: (context, AsyncSnapshot<DocumentSnapshot> snapshot) {
            if (snapshot.hasData) {
              FigureModel figureModel =
              FigureModel.fromDocument(snapshot.data!);
              return Container(
                  child: Scaffold(
                      appBar: AppPageBar(
                        leadingText: AppTexts
                            .activityAvailablePeopleBackMenuItem,
                        barActions: [
                          IconButton(onPressed: (){
                            Navigator.push(
                              context,
                              MaterialPageRoute(builder: (context) => ActivityFigureAddPeople(activityModel: activityModel,activityFigureModel: activityFigureModel)),
                            ).then((value) => setState((){}));
                          }, icon: AppIcons.activityFigureAssignPeopleAdd),
                          IconButton(onPressed: (){
                            Navigator.push(
                              context,
                              MaterialPageRoute(builder: (context) => ActivityFigureRemovePeople(activityModel: activityModel,activityFigureModel: activityFigureModel)),
                            ).then((value) => setState((){}));
                          }, icon: AppIcons.activityFigureAssignPeopleView),
                        ],
                      ),
                      body: Container(
                        child: Column(children: [
                          ActivityFigureInfoBar(activityModel: activityModel, activityFigureModel: activityFigureModel),
                          SizedBox(height: 10),
                          Flexible(
                            fit: FlexFit.tight,
                              child: ActivityFigureDiagram(figureModel: figureModel, activityFigureModel: activityFigureModel),
                          ),
                          SizedBox(height: 10),
                        ]),
                      )),
                );
            } else if (snapshot.hasError) {
              return Text(snapshot.error.toString());
            }
            return Center(child: CircularProgressIndicator());
          }
          );
    } else
      return NoAccessNoLoggedInUser();
  }

  Widget buildItem(BuildContext context, DocumentSnapshot? document) {
    if (document != null) {
      AttendeeModel attendeeModel = AttendeeModel.fromDocument(document);
      return Container(
          padding: EdgeInsets.all(10),
          height: 70,
          decoration: BoxDecoration(
            border: Border.all(width: 1, color: AppColours.midGray),
          ),
          child: Row(
            children: [
              AppIcons.standardUserAttendance,
              Flexible(
                fit: FlexFit.tight,
                child: Text(
                  attendeeModel.nickName,
                  style: AppStyles.activityAttendanceTextStyle,
                ),
              ),
            ],
          ));
    } else {
      return SizedBox.shrink();
    }
  }
}
