/// MuixApp
///
/// Prototipus d’aplicació mòbil Android/iOS per a la gestió d’assajos, actuacions i catàlegs de figures de colles muixerangueres.
///
/// Grau en Enginyeria Informàtica.
/// Treball Final de Grau. Àrea Desenvolupament multiplataforma d'aplicacions mòbils.
/// Universitat Oberta de Catalunya.
/// Curs 2021-22.
///
/// Estudiant: Hèctor Rubio i Marín (hrubio@uoc.edu)
///
/// MIT License
/// Copyright (c) 2021 Hèctor Rubio i Marín

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';
import 'package:muixapp/src/models/activity_figure_model.dart';
import 'package:muixapp/src/models/activity_model.dart';
import 'package:muixapp/src/models/attendee_model.dart';
import 'package:muixapp/src/providers/providers.dart';
import 'package:muixapp/src/ui/resources/resources.dart';
import 'package:muixapp/src/ui/widgets/activities/activity_figure_info_bar.dart';
import 'package:muixapp/src/ui/widgets/authentication/no_access_no_logged_in_user.dart';
import 'package:muixapp/src/ui/widgets/common/app_page_bar.dart';
import 'package:provider/provider.dart';

class ActivityFigureRemovePeople extends StatefulWidget {
  final ActivityModel activityModel;
  final ActivityFigureModel activityFigureModel;

  const ActivityFigureRemovePeople(
      {required this.activityModel,
      required this.activityFigureModel,
      Key? key})
      : super(key: key);

  @override
  _ActivityFigureRemovePeopleState createState() => _ActivityFigureRemovePeopleState(
      this.activityModel, this.activityFigureModel);
}

class _ActivityFigureRemovePeopleState extends State<ActivityFigureRemovePeople> {
  final ActivityModel activityModel;
  final ActivityFigureModel activityFigureModel;

  _ActivityFigureRemovePeopleState(this.activityModel, this.activityFigureModel);

  late AttendeesProvider attendeesProvider;
  late AuthenticationProvider authenticationProvider;
  late ActivityFiguresProvider activityFiguresProvider;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    authenticationProvider = Provider.of<AuthenticationProvider>(context);
    attendeesProvider = context.read<AttendeesProvider>();
    activityFiguresProvider = context.read<ActivityFiguresProvider>();
  }

  @override
  Widget build(BuildContext context) {
    if (authenticationProvider.isLoggedIn())
      return Container(
        child: Scaffold(
            appBar: AppPageBar(
              leadingText: AppTexts.backAppMainBarNavigation,
              barActions: [],
            ),
            body: Container(
              child: Column(children: [
                ActivityFigureInfoBar(activityModel: activityModel, activityFigureModel: activityFigureModel,),
                SizedBox(height: 10),
                Flexible(
                  fit: FlexFit.tight,
                  child: StreamBuilder<QuerySnapshot>(
                    stream:
                        attendeesProvider.getAvailableAttendeesStreamByActivity(
                            this.activityModel.id),
                    builder: (context, AsyncSnapshot<QuerySnapshot> snapshot) {
                      if (snapshot.hasData) {
                        int counter = (snapshot.data?.docs.length ?? 0);
                        if (counter > 0) {
                          // return Text(counter.toString());
                          return ListView.builder(
                            padding: EdgeInsets.symmetric(horizontal: 8),
                            itemBuilder: (context, index) => buildItem(
                              context,
                              activityModel,
                              activityFigureModel,
                              snapshot.data?.docs[index],
                            ),
                            itemCount: snapshot.data?.docs.length,
                          );
                        } else
                          return Center(
                              child: Container(
                                  margin: EdgeInsets.all(20),
                                  child: Text(AppTexts
                                      .activityAvailablePeopleNoPeopleMenuItem)));
                      } else if (snapshot.hasError) {
                        return Text(snapshot.error.toString());
                      }
                      return Center(child: CircularProgressIndicator());
                    },
                  ),
                ),
                SizedBox(height: 10),
              ]),
            )),
      );
    else
      return NoAccessNoLoggedInUser();
  }

  Widget buildItem(BuildContext context, ActivityModel activityModel, ActivityFigureModel activityFigureModel, DocumentSnapshot? document) {
    if (document != null) {
      AttendeeModel attendeeModel = AttendeeModel.fromDocument(document);
      if (activityFigureModel.attendees.containsKey(attendeeModel.id))
        return Container(
          padding: EdgeInsets.fromLTRB(10, 10, 0, 10),
          height: 70,
          decoration: BoxDecoration(
            border: Border.all(width: 1, color: AppColours.midGray),
          ),
          child: Row(
            children: [
              AppIcons.standardUserAttendance,
              Flexible(
                fit: FlexFit.tight,
                child: Column(
                    mainAxisAlignment: MainAxisAlignment.start,
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Text(
                        attendeeModel.nickName,
                        style: AppStyles
                            .activityFiguresAssignPeopleNickNameTimeStyle,
                      ),
                      SizedBox(
                        height: 5,
                      ),
                      Text(
                        (attendeeModel.usedIn == 1
                            ? attendeeModel.usedIn.toString() + ' '+AppTexts.activityFigureAddRemovePeopleSingular
                            : attendeeModel.usedIn.toString() + ' '+AppTexts.activityFigureAddRemovePeoplePlural),
                        style: AppStyles
                            .activityFiguresAssignPeopleUsedInTimeStyle,
                      ),
                    ]),
              ),
              IconButton(
                  onPressed: () async {
                    if (await attendeesProvider.decreaseAttendeeUsedIn(
                        attendeeModel)){
                      if (await activityFiguresProvider.removeAttendee(activityFigureModel, attendeeModel))
                        Navigator.pop(context);
                    }
                  },
                  icon: AppIcons.activityFigureAssignPeopleRemove),
            ],
          ));
      else
        return SizedBox.shrink();
    } else {
      return SizedBox.shrink();
    }
  }
}


