/// MuixApp
///
/// Prototipus d’aplicació mòbil Android/iOS per a la gestió d’assajos, actuacions i catàlegs de figures de colles muixerangueres.
///
/// Grau en Enginyeria Informàtica.
/// Treball Final de Grau. Àrea Desenvolupament multiplataforma d'aplicacions mòbils.
/// Universitat Oberta de Catalunya.
/// Curs 2021-22.
///
/// Estudiant: Hèctor Rubio i Marín (hrubio@uoc.edu)
///
/// MIT License
/// Copyright (c) 2021 Hèctor Rubio i Marín

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';
import 'package:muixapp/src/models/activity_figure_model.dart';
import 'package:muixapp/src/models/activity_model.dart';
import 'package:muixapp/src/providers/providers.dart';
import 'package:muixapp/src/ui/resources/resources.dart';
import 'package:muixapp/src/ui/widgets/activities/activity_figure_options_action.dart';
import 'package:muixapp/src/ui/widgets/activities/activity_figures_list_options_action.dart';
import 'package:muixapp/src/ui/widgets/activities/activity_info_bar.dart';
import 'package:muixapp/src/ui/widgets/authentication/no_access_no_logged_in_user.dart';
import 'package:muixapp/src/ui/widgets/common/app_page_bar.dart';
import 'package:provider/provider.dart';

class ActivityFiguresList extends StatefulWidget {
  final ActivityModel activityModel;

  const ActivityFiguresList({required this.activityModel, Key? key})
      : super(key: key);

  @override
  _ActivityFiguresListState createState() =>
      _ActivityFiguresListState(this.activityModel);
}

class _ActivityFiguresListState extends State<ActivityFiguresList> {
  final ActivityModel activityModel;

  _ActivityFiguresListState(this.activityModel);

  late ActivityFiguresProvider activityFiguresProvider;
  late AuthenticationProvider authenticationProvider;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    authenticationProvider = Provider.of<AuthenticationProvider>(context);
    activityFiguresProvider = context.read<ActivityFiguresProvider>();
  }

  @override
  Widget build(BuildContext context) {
    if (authenticationProvider.isLoggedIn())
      return Container(
        child: Scaffold(
            appBar: AppPageBar(
              leadingText: AppTexts.activityPageTitle,
              barActions: [ActivityFiguresListOptionsAction(activityModel: this.activityModel)],            ),
            body: Container(
              child: Column(children: [
                ActivityInfoBar(activityModel: activityModel),
                SizedBox(height: 10),
                Flexible(
                  fit: FlexFit.tight,
                  child: StreamBuilder<QuerySnapshot>(
                    stream: activityFiguresProvider
                        .getActivityFiguresStreamByActivity(
                            this.activityModel.id),
                    builder: (context, AsyncSnapshot<QuerySnapshot> snapshot) {
                      if (snapshot.hasData) {
                        int counter = (snapshot.data?.docs.length ?? 0);
                        if (counter > 0) {
                          // return Text(counter.toString());
                          return ListView.builder(
                            padding: EdgeInsets.symmetric(horizontal: 8),
                            itemBuilder: (context, index) => buildItem(
                              context,
                              activityModel,
                              snapshot.data?.docs[index],
                            ),
                            itemCount: snapshot.data?.docs.length,
                          );
                        } else
                          return Center(
                              child: Container(
                                  margin: EdgeInsets.all(20),
                                  child:Text(AppTexts.activityFiguresNoFiguresMenuItem))
                          );
                      } else if (snapshot.hasError) {
                        return Text(snapshot.error.toString());
                      }
                      return Center(child: CircularProgressIndicator());
                    },
                  ),
                ),
                SizedBox(height: 10),
              ]),
            )),
      );
    else
      return NoAccessNoLoggedInUser();
  }

  Widget buildItem(BuildContext context, ActivityModel activityModel, DocumentSnapshot? document) {
    if (document != null) {

      ActivityFigureModel activityFigureModel =
          ActivityFigureModel.fromDocument(document);
      return Container(
          padding: EdgeInsets.all(10),
          height: 90,
          decoration: BoxDecoration(
            border: Border.all(width: 1, color: AppColours.midGray),
          ),
          child: Row(
            children: [
              AppIcons.activityFigure,
              Flexible(
                fit: FlexFit.tight,
                child: Container(
                    margin: EdgeInsets.symmetric(vertical: 5.0),
                    height: 70,
                    child: Column(
                        mainAxisAlignment: MainAxisAlignment.start,
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          Text(
                            activityFigureModel.name.toUpperCase(),
                            style: AppStyles.activityFiguresListNameTextStyle,
                          ),
                          SizedBox(height: 5),
                          (activityFigureModel.version.isNotEmpty
                              ? Text(
                                  activityFigureModel.version,
                                  overflow: TextOverflow.ellipsis,
                                  maxLines: 1)
                              : SizedBox.shrink()),
                          Text(activityFigureModel.team),
                        ])),
              ),
              ActivityFigureOptionsAction(activityModel: activityModel, activityFigureModel: activityFigureModel),
            ],
          ));
    } else {
      return SizedBox.shrink();
    }
  }
}
