/// MuixApp
///
/// Prototipus d’aplicació mòbil Android/iOS per a la gestió d’assajos, actuacions i catàlegs de figures de colles muixerangueres.
///
/// Grau en Enginyeria Informàtica.
/// Treball Final de Grau. Àrea Desenvolupament multiplataforma d'aplicacions mòbils.
/// Universitat Oberta de Catalunya.
/// Curs 2021-22.
///
/// Estudiant: Hèctor Rubio i Marín (hrubio@uoc.edu)
///
/// MIT License
/// Copyright (c) 2021 Hèctor Rubio i Marín

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';
import 'package:muixapp/src/models/profile_model.dart';
import 'package:muixapp/src/providers/authentication_provider.dart';
import 'package:muixapp/src/providers/profiles_provider.dart';
import 'package:muixapp/src/providers/providers.dart';
import 'package:muixapp/src/ui/pages/authentication/register.dart';
import 'package:muixapp/src/ui/pages/home_page.dart';
import 'package:muixapp/src/ui/resources/resources.dart';
import 'package:provider/provider.dart';

class Login extends StatefulWidget {
  const Login({Key? key}) : super(key: key);

  @override
  _LoginState createState() => _LoginState();
}

class _LoginState extends State<Login> {
  final emailTextEditingController = new TextEditingController();
  final passwordTextEditingController = new TextEditingController();

  late AuthenticationProvider authenticationProvider;
  late ProfilesProvider profilesProvider;
  late PreferencesProvider preferencesProvider;

  late ProfileModel profileModel;

  @override
  void initState() {
    super.initState();
  }

  @override
  void dispose() {
    emailTextEditingController.dispose();
    passwordTextEditingController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    authenticationProvider = Provider.of<AuthenticationProvider>(context);
    profilesProvider = Provider.of<ProfilesProvider>(context);
    preferencesProvider = Provider.of<PreferencesProvider>(context);

    return new GestureDetector(
      onTap: () => FocusManager.instance.primaryFocus?.unfocus(),
      child: Scaffold(
      resizeToAvoidBottomInset: false,
      backgroundColor: AppColours.white,
      body: SafeArea(
        child: Container(
          color: AppColours.midGray,
          margin: EdgeInsets.all(10.0),
          child: Center(
              child: Column(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              Text(
                AppTexts.appTitle,
                style: AppStyles.appTitleTextStyle,
              ),
              SizedBox(
                height: 100,
              ),
              SizedBox(
                height: 250,
                width: 230,
                child: Container(
                  color: AppColours.lightGray,
                  padding: EdgeInsets.all(20),
                  child: Column(
                    children: [
                      // email field
                      TextField(
                        controller: emailTextEditingController,
                        style: TextStyle(
                          fontSize: 14,
                          height: 1,
                        ),
                        decoration: InputDecoration(
                            filled: true,
                            fillColor: AppColours.white,
                            hintText: AppTexts.registerEmailHint,
                            isDense: true,
                            contentPadding: EdgeInsets.symmetric(
                              vertical: 8,
                              horizontal: 8,
                            ),
                            border: OutlineInputBorder(
                              borderRadius: BorderRadius.all(Radius.zero),
                            )),
                        enableSuggestions: true,
                        autocorrect: false,
                      ),
                      SizedBox(
                        height: 20,
                      ),

                      // password field
                      TextField(
                        controller: passwordTextEditingController,
                        style: TextStyle(
                          fontSize: 14,
                          height: 1,
                        ),
                        decoration: InputDecoration(
                            filled: true,
                            fillColor: AppColours.white,
                            hintText: AppTexts.registerPasswordHint,
                            isDense: true,
                            contentPadding: EdgeInsets.symmetric(
                              vertical: 8,
                              horizontal: 8,
                            ),
                            border: OutlineInputBorder(
                              borderRadius: BorderRadius.all(Radius.zero),
                            )),
                        obscureText: true,
                        obscuringCharacter: '*',
                        enableSuggestions: false,
                        autocorrect: false,
                      ),
                      SizedBox(
                        height: 40,
                      ),

                      // login button
                      TextButton(
                        style: TextButton.styleFrom(
                          backgroundColor: AppColours.blue,
                          minimumSize: Size(230, 30),
                        ),
                        onPressed: () async {
                          if (await authenticationProvider.login(emailTextEditingController.text, passwordTextEditingController.text, context)) {
                            Future<DocumentSnapshot> doc = profilesProvider
                                .getProfile(
                                preferencesProvider.getKeyStringValue('uid')!);
                            doc.then((value) =>
                            {
                              setLoggegInSettings(ProfileModel.fromDocument(value))
                              /*preferencesProvider.setKeyStringValue(
                                  'usernickname', ProfileModel
                                  .fromDocument(value)
                                  .nickName)*/
                            });

                            Navigator.pushReplacement(
                              context,
                              MaterialPageRoute(
                                  builder: (context) => HomePage()),
                            );
                          }
                        },
                        child: Text(
                          AppTexts.loginLoginButtonCaption,
                          style: TextStyle(
                            fontSize: 16,
                            fontWeight: FontWeight.bold,
                            color: AppColours.white,
                            height: 1,
                          ),
                        ),
                      )
                    ],
                  ),
                ),
              ),
              SizedBox(
                height: 40,
              ),

              // register button
              TextButton(
                style: TextButton.styleFrom(
                  backgroundColor: AppColours.lightBlue,
                  minimumSize: Size(190, 30),
                ),
                onPressed: () {
                  Navigator.push(
                    context,
                    MaterialPageRoute(builder: (context) => Register()),
                  );
                },
                child: Text(
                  AppTexts.loginRegisterButtonCaption,
                  style: TextStyle(
                    fontSize: 16,
                    fontWeight: FontWeight.bold,
                    color: AppColours.white,
                    height: 1,
                  ),
                ),
              ),
            ],
          )),
        ),
      ),
    ),
    );
  }

  void setLoggegInSettings(ProfileModel profileModel){
    preferencesProvider.setKeyStringValue('usernickname', profileModel.nickName);
    preferencesProvider.setKeyBoolValue('ismanager', profileModel.isManager);
    preferencesProvider.setKeyBoolValue('istrainer', profileModel.isTrainer);
  }
}
