/// MuixApp
///
/// Prototipus d’aplicació mòbil Android/iOS per a la gestió d’assajos, actuacions i catàlegs de figures de colles muixerangueres.
///
/// Grau en Enginyeria Informàtica.
/// Treball Final de Grau. Àrea Desenvolupament multiplataforma d'aplicacions mòbils.
/// Universitat Oberta de Catalunya.
/// Curs 2021-22.
///
/// Estudiant: Hèctor Rubio i Marín (hrubio@uoc.edu)
///
/// MIT License
/// Copyright (c) 2021 Hèctor Rubio i Marín

import 'package:flutter/material.dart';
import 'package:muixapp/src/models/profile_model.dart';
import 'package:muixapp/src/providers/providers.dart';
import 'package:muixapp/src/ui/pages/home_page.dart';
import 'package:muixapp/src/ui/resources/resources.dart';
import 'package:provider/provider.dart';

class Register extends StatefulWidget {
  const Register({Key? key}) : super(key: key);

  @override
  _RegisterState createState() => _RegisterState();
}

class _RegisterState extends State<Register> {

  final firstNameTextEditingController = new TextEditingController();
  final lastNameTextEditingController = new TextEditingController();
  final nickNameTextEditingController = new TextEditingController();
  final mobilePhoneTextEditingController = new TextEditingController();
  final emailTextEditingController = new TextEditingController();
  final passwordTextEditingController = new TextEditingController();
  final repeatPasswordTextEditingController = new TextEditingController();

  late AuthenticationProvider authenticationProvider;
  late ProfilesProvider profilesProvider;
  late PreferencesProvider preferencesProvider;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    authenticationProvider = Provider.of<AuthenticationProvider>(context);
    profilesProvider = Provider.of<ProfilesProvider>(context);
    preferencesProvider = Provider.of<PreferencesProvider>(context);
  }

  @override
  void dispose() {
    firstNameTextEditingController.dispose();
    lastNameTextEditingController.dispose();
    nickNameTextEditingController.dispose();
    mobilePhoneTextEditingController.dispose();
    emailTextEditingController.dispose();
    passwordTextEditingController.dispose();
    repeatPasswordTextEditingController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {

    return new GestureDetector(
      onTap: () => FocusManager.instance.primaryFocus?.unfocus(),
      child: Scaffold(
        resizeToAvoidBottomInset: false,
        backgroundColor: AppColours.white,
        body: SafeArea(
          child: Container(
            color: AppColours.midGray,
            margin: EdgeInsets.all(10.0),
            child: Center(
                child: Column(
              mainAxisAlignment: MainAxisAlignment.start,
              children: [
                SizedBox(
                  height: 60,
                ),

                // app title
                Text(
                  AppTexts.appTitle,
                  style: AppStyles.appTitleTextStyle,
                ),
                SizedBox(
                  height: 10,
                ),

                // form box
                SizedBox(
                  height: 520,
                  width: 230,
                  child: Container(
                    color: AppColours.lightGray,
                    padding: EdgeInsets.fromLTRB(20, 0, 20, 0),
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      mainAxisAlignment: MainAxisAlignment.center,
                      children: [

                        // first name field
                        Padding(
                          padding: const EdgeInsets.symmetric(vertical: 8.0),
                          child: Text(AppTexts.registerFirstName),
                        ),
                        TextField(
                          controller: firstNameTextEditingController,
                          style: TextStyle(
                            fontSize: 14,
                            height: 1,
                          ),
                          decoration: InputDecoration(
                              filled: true,
                              fillColor: AppColours.white,
                              hintText: AppTexts.registerFirstNameHint,
                              isDense: true,
                              contentPadding: EdgeInsets.symmetric(
                                vertical: 8,
                                horizontal: 8,
                              ),
                              border: OutlineInputBorder(
                                borderRadius: BorderRadius.all(Radius.zero),
                              )),
                          enableSuggestions: true,
                          autocorrect: false,
                        ),

                        // last name field
                        Padding(
                          padding: const EdgeInsets.symmetric(vertical: 8.0),
                          child: Text(AppTexts.registerLastName),
                        ),
                        TextField(
                          controller: lastNameTextEditingController,
                          style: TextStyle(
                            fontSize: 14,
                            height: 1,
                          ),
                          decoration: InputDecoration(
                              filled: true,
                              fillColor: AppColours.white,
                              hintText: AppTexts.registerLastNameHint,
                              isDense: true,
                              contentPadding: EdgeInsets.symmetric(
                                vertical: 8,
                                horizontal: 8,
                              ),
                              border: OutlineInputBorder(
                                borderRadius: BorderRadius.all(Radius.zero),
                              )),
                          enableSuggestions: true,
                          autocorrect: false,
                        ),

                        // nickname field
                        Padding(
                          padding: const EdgeInsets.symmetric(vertical: 8.0),
                          child: Text(AppTexts.registerNickName),
                        ),
                        TextField(
                          controller: nickNameTextEditingController,
                          style: TextStyle(
                            fontSize: 14,
                            height: 1,
                          ),
                          decoration: InputDecoration(
                              filled: true,
                              fillColor: AppColours.white,
                              hintText: AppTexts.registerNickNameHint,
                              isDense: true,
                              contentPadding: EdgeInsets.symmetric(
                                vertical: 8,
                                horizontal: 8,
                              ),
                              border: OutlineInputBorder(
                                borderRadius: BorderRadius.all(Radius.zero),
                              )),
                          enableSuggestions: true,
                          autocorrect: false,
                        ),

                        // mobile phone field
                        Padding(
                          padding: const EdgeInsets.symmetric(vertical: 8.0),
                          child: Text(AppTexts.registerMobilePhone),
                        ),
                        TextField(
                          controller: mobilePhoneTextEditingController,
                          style: TextStyle(
                            fontSize: 14,
                            height: 1,
                          ),
                          decoration: InputDecoration(
                              filled: true,
                              fillColor: AppColours.white,
                              hintText: AppTexts.registerMobilePhoneHint,
                              isDense: true,
                              contentPadding: EdgeInsets.symmetric(
                                vertical: 8,
                                horizontal: 8,
                              ),
                              border: OutlineInputBorder(
                                borderRadius: BorderRadius.all(Radius.zero),
                              )),
                          enableSuggestions: true,
                          autocorrect: false,
                        ),

                        // email field
                        Padding(
                          padding: const EdgeInsets.symmetric(vertical: 8.0),
                          child: Text(AppTexts.registerEmail),
                        ),
                        TextField(
                          controller: emailTextEditingController,
                          style: TextStyle(
                            fontSize: 14,
                            height: 1,
                          ),
                          decoration: InputDecoration(
                              filled: true,
                              fillColor: AppColours.white,
                              hintText: AppTexts.registerEmailHint,
                              isDense: true,
                              contentPadding: EdgeInsets.symmetric(
                                vertical: 8,
                                horizontal: 8,
                              ),
                              border: OutlineInputBorder(
                                borderRadius: BorderRadius.all(Radius.zero),
                              )),
                          enableSuggestions: true,
                          autocorrect: false,
                        ),

                        // password field
                        Padding(
                          padding: const EdgeInsets.symmetric(vertical: 8.0),
                          child: Text(AppTexts.registerPassword),
                        ),
                        TextField(
                          controller: passwordTextEditingController,
                          style: TextStyle(
                            fontSize: 14,
                            height: 1,
                          ),
                          decoration: InputDecoration(
                              filled: true,
                              fillColor: AppColours.white,
                              hintText: AppTexts.registerPasswordHint,
                              isDense: true,
                              contentPadding: EdgeInsets.symmetric(
                                vertical: 8,
                                horizontal: 8,
                              ),
                              border: OutlineInputBorder(
                                borderRadius: BorderRadius.all(Radius.zero),
                              )),
                          obscureText: true,
                          obscuringCharacter: '*',
                          enableSuggestions: false,
                          autocorrect: false,
                        ),

                        // repeat password field
                        Padding(
                          padding: const EdgeInsets.symmetric(vertical: 8.0),
                          child: Text(AppTexts.registerRepeatPassword),
                        ),
                        TextField(
                          controller: repeatPasswordTextEditingController,
                          style: TextStyle(
                            fontSize: 14,
                            height: 1,
                          ),
                          decoration: InputDecoration(
                              filled: true,
                              fillColor: AppColours.white,
                              hintText: AppTexts.registerRepeatPasswordHint,
                              isDense: true,
                              contentPadding: EdgeInsets.symmetric(
                                vertical: 8,
                                horizontal: 8,
                              ),
                              border: OutlineInputBorder(
                                borderRadius: BorderRadius.all(Radius.zero),
                              )),
                          obscureText: true,
                          obscuringCharacter: '*',
                          enableSuggestions: false,
                          autocorrect: false,
                        ),

                        SizedBox(
                          height: 20,
                        ),
                        // register button
                        TextButton(
                          style: TextButton.styleFrom(
                            backgroundColor: AppColours.blue,
                            minimumSize: Size(230, 30),
                          ),
                          onPressed: () async {
                            if (validateForm()) {
                              if (await authenticationProvider.register(
                                  emailTextEditingController.text,
                                  passwordTextEditingController.text, context)) {

                                ProfileModel profile = new ProfileModel(
                                    id: preferencesProvider.getKeyStringValue('uid')!,
                                    firstName: firstNameTextEditingController.text,
                                    lastName: lastNameTextEditingController.text,
                                    nickName: nickNameTextEditingController.text,
                                    mobilePhone: mobilePhoneTextEditingController.text,
                                    email:preferencesProvider.getKeyStringValue('useremail')!,
                                    isManager: false,
                                    isTrainer: false,
                                    bio: '');

                                preferencesProvider.setKeyStringValue("usernickname", nickNameTextEditingController.text);
                                
                                if (await profilesProvider.addProfile(profile))
                                  Navigator.pushReplacement(
                                    context,
                                    MaterialPageRoute(
                                        builder: (context) => HomePage()),
                                  );
                              }
                            }
                          },
                          child: Text(
                            AppTexts.registerRegisterButtonCaption,
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.bold,
                              color: AppColours.white,
                              height: 1,
                            ),
                          ),
                        )
                      ],
                    ),
                  ),
                ),
              ],
            )),
          ),
        ),
      ),
    );
  }

  bool validateForm() {
    //TODO: Enhance validation of nickName (should be unique in each team), mobilePhone

    String errorMessage = '';

    if (passwordTextEditingController.text!=repeatPasswordTextEditingController.text){
      errorMessage = AppTexts.authenticationDifferentPasswords;
    } else if (firstNameTextEditingController.text.isEmpty){
      errorMessage = AppTexts.authenticationEmptyFirstName;
    } else if (lastNameTextEditingController.text.isEmpty){
      errorMessage = AppTexts.authenticationEmptyLastName;
    } else if (nickNameTextEditingController.text.isEmpty){
      errorMessage = AppTexts.authenticationEmptyNickName;
    } else if (mobilePhoneTextEditingController.text.isEmpty){
      errorMessage = AppTexts.authenticationEmptyMobilePhone;
    }

    if (errorMessage.isEmpty)
      return true;
    else {
      ScaffoldMessenger.of(context).showSnackBar(
        SnackBar(
          content: Container(child:Text(errorMessage)),
          padding: EdgeInsets.all(20),
        ),
      );
      return false;
    }
  }
}
