/// MuixApp
///
/// Prototipus d’aplicació mòbil Android/iOS per a la gestió d’assajos, actuacions i catàlegs de figures de colles muixerangueres.
///
/// Grau en Enginyeria Informàtica.
/// Treball Final de Grau. Àrea Desenvolupament multiplataforma d'aplicacions mòbils.
/// Universitat Oberta de Catalunya.
/// Curs 2021-22.
///
/// Estudiant: Hèctor Rubio i Marín (hrubio@uoc.edu)
///
/// MIT License
/// Copyright (c) 2021 Hèctor Rubio i Marín

import 'package:flutter/material.dart';
import 'package:muixapp/src/models/figure_model.dart';
import 'package:muixapp/src/ui/resources/app_styles.dart';
import 'package:muixapp/src/ui/resources/resources.dart';
import 'package:muixapp/src/ui/widgets/authentication/no_access_no_logged_in_user.dart';
import 'package:muixapp/src/ui/widgets/common/app_page_bar.dart';
import 'package:provider/provider.dart';
import 'package:muixapp/src/ui/resources/app_colours.dart';
import 'package:muixapp/src/ui/resources/app_texts.dart';
import 'package:muixapp/src/providers/providers.dart';
import 'package:url_launcher/url_launcher.dart';

class FigureVideo extends StatefulWidget {
  final FigureModel figureModel;
  const FigureVideo({required this.figureModel, Key? key}) : super(key: key);

  @override
  _FigureVideoState createState() => _FigureVideoState(this.figureModel);
}

class _FigureVideoState extends State<FigureVideo> {
  final FigureModel figureModel;

  _FigureVideoState(this.figureModel);

  late AuthenticationProvider authenticationProvider;
  late FiguresProvider figuresProvider;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    authenticationProvider = Provider.of<AuthenticationProvider>(context);
    figuresProvider = Provider.of<FiguresProvider>(context);
  }

  @override
  Widget build(BuildContext context) {
    if (authenticationProvider.isLoggedIn())
      return Scaffold(
        appBar: AppPageBar(
          leadingText: AppTexts.catalogueFigureNavigation,
          barActions: [],
        ),
        body: Container(
          margin: EdgeInsets.fromLTRB(10, 20, 10, 10),
          child: Column(
            mainAxisAlignment: MainAxisAlignment.start,
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
// Logo, name..

              Container(
                padding: EdgeInsets.all(10),
                decoration: BoxDecoration(
                  color: AppColours.lightGray,
                ),
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    Row(
                      children: [
                        // Left column - Icon
                        AppIcons.activityFigure,
                        Flexible(
                          fit: FlexFit.tight,
                          child: Container(
                              margin: EdgeInsets.symmetric(vertical: 5.0),
                              height: 70,
                              child: Column(
                                  mainAxisAlignment: MainAxisAlignment.start,
                                  crossAxisAlignment: CrossAxisAlignment.start,
                                  children: [
                                    Text(
                                      figureModel.name.toUpperCase(),
                                      style: AppStyles
                                          .activityAddFiguresListNameTextStyle,
                                    ),
                                    SizedBox(height: 5),
                                    (figureModel.version.isNotEmpty
                                        ? Text(figureModel.version,
                                            overflow: TextOverflow.ellipsis,
                                            maxLines: 1)
                                        : SizedBox.shrink()),
                                    Text(figureModel.team),
                                  ])),
                        ),
                      ],
                    ),
                  ],
                  //color: AppColours.white,
                ),
              ),

              // Figure video

              (figureModel.videoURL.isNotEmpty
                  ? TextButton(
                      onPressed: () async {
                        await launch(figureModel.videoURL);
                      },
                      child: Text(figureModel.videoURL,
                          style: AppStyles.standardUnderlinedTextStyle))
                  : SizedBox.shrink()),
            ],
          ),
        ),
      );
    else
      return NoAccessNoLoggedInUser();
  }
}
