/// MuixApp
///
/// Prototipus d’aplicació mòbil Android/iOS per a la gestió d’assajos, actuacions i catàlegs de figures de colles muixerangueres.
///
/// Grau en Enginyeria Informàtica.
/// Treball Final de Grau. Àrea Desenvolupament multiplataforma d'aplicacions mòbils.
/// Universitat Oberta de Catalunya.
/// Curs 2021-22.
///
/// Estudiant: Hèctor Rubio i Marín (hrubio@uoc.edu)
///
/// MIT License
/// Copyright (c) 2021 Hèctor Rubio i Marín

import 'package:flutter/material.dart';
import 'package:muixapp/src/ui/pages/pages.dart';
import 'package:muixapp/src/ui/resources/resources.dart';
import 'package:muixapp/src/ui/widgets/common/app_main_bar.dart';

class HomePage extends StatefulWidget {
  const HomePage({Key? key}) : super(key: key);

  @override
  _HomePageState createState() => _HomePageState();
}

class _HomePageState extends State<HomePage> {
  List<Widget> _pages = [];
  int _currentIndex = 0;
  Widget _currentPage = ActivitiesList();

  @override
  void initState() {
    super.initState();
    _pages = [
      ActivitiesList(),
      PeopleList(),
      FiguresList(),
      MessagesList(),
      Profile()
    ];
    _currentIndex = 0;
    _currentPage = _pages[0];
  }

  void changeTab(int index) {
    setState(() {
      _currentIndex = index;
      _currentPage = _pages[index];
    });
  }

  @override
  Widget build(BuildContext context) {
    return new GestureDetector(
      onTap: () => FocusManager.instance.primaryFocus?.unfocus(),
      child: Scaffold(
        resizeToAvoidBottomInset: false,
        appBar: AppMainBar(),
        body: _currentPage,
        backgroundColor: AppColours.white,
        bottomNavigationBar: BottomNavigationBar(
            type: BottomNavigationBarType.fixed,
            onTap: (index) => changeTab(index),
            currentIndex: _currentIndex,
            items: [
              BottomNavigationBarItem(
                icon: AppIcons.mainNavigationActivities,
                label: AppTexts.activitiesNavigation,
              ),
              BottomNavigationBarItem(
                icon: AppIcons.mainNavigationPeople,
                label: AppTexts.teamNavigation,
              ),
              BottomNavigationBarItem(
                icon: AppIcons.mainNavigationFigures,
                label: AppTexts.catalogueNavigation,
              ),
              BottomNavigationBarItem(
                icon: AppIcons.mainNavigationMessages,
                label: AppTexts.messagesNavigation,
              ),
              BottomNavigationBarItem(
                icon: AppIcons.mainNavigationProfile,
                label: AppTexts.profileNavigation,
              ),
            ]),
      ),
    );
  }
}
