/// MuixApp
///
/// Prototipus d’aplicació mòbil Android/iOS per a la gestió d’assajos, actuacions i catàlegs de figures de colles muixerangueres.
///
/// Grau en Enginyeria Informàtica.
/// Treball Final de Grau. Àrea Desenvolupament multiplataforma d'aplicacions mòbils.
/// Universitat Oberta de Catalunya.
/// Curs 2021-22.
///
/// Estudiant: Hèctor Rubio i Marín (hrubio@uoc.edu)
///
/// MIT License
/// Copyright (c) 2021 Hèctor Rubio i Marín

import 'package:flutter/material.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:muixapp/src/ui/resources/resources.dart';
import 'package:muixapp/src/ui/widgets/authentication/no_access_no_logged_in_user.dart';
import 'package:muixapp/src/ui/widgets/common/app_page_bar.dart';
import 'package:provider/provider.dart';
import 'package:muixapp/src/models/notification_model.dart';
import 'package:muixapp/src/providers/providers.dart';
import 'package:muixapp/src/ui/widgets/notifications/notifications_list_tile.dart';

class NotificationsList extends StatefulWidget {
  NotificationsList({Key? key}) : super(key: key);

  @override
  State createState() => NotificationsListState();
}

class NotificationsListState extends State<NotificationsList> {
  NotificationsListState({Key? key});

  TextEditingController searchController = TextEditingController();

  late NotificationsProvider notificationsProvider;
  late AuthenticationProvider authenticationProvider;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    authenticationProvider = Provider.of<AuthenticationProvider>(context);
    notificationsProvider = context.read<NotificationsProvider>();
  }

  @override
  Widget build(BuildContext context) {
    if (authenticationProvider.isLoggedIn())
      return Container(
        child: Scaffold(
            appBar: AppPageBar(
              leadingText: AppTexts.backAppMainBarNavigation,
              barActions: [],
            ),
            body: Container(
              child: Column(children: [
                SizedBox(height: 10),
                Flexible(
                  fit: FlexFit.tight,
                  child: StreamBuilder<QuerySnapshot>(
                    stream:
                        notificationsProvider.getActiveNotificationsStream(),
                    builder: (context, AsyncSnapshot<QuerySnapshot> snapshot) {
                      if (snapshot.hasData) {
                        int counter = (snapshot.data?.docs.length ?? 0);
                        if (counter > 0) {
                          return ListView.builder(
                            padding: EdgeInsets.symmetric(horizontal: 8),
                            itemBuilder: (context, index) => buildItem(
                              context,
                              snapshot.data?.docs[index],
                            ),
                            itemCount: snapshot.data?.docs.length,
                          );
                        } else
                          return SizedBox.shrink();
                      } else if (snapshot.hasError) {
                        return Text(snapshot.error.toString());
                      }
                      return Center(child: CircularProgressIndicator());
                    },
                  ),
                ),
                SizedBox(height: 10),
              ]),
            )),
      );
    else
      return NoAccessNoLoggedInUser();
  }

  Widget buildItem(BuildContext context, DocumentSnapshot? document) {
    if (document != null) {
      NotificationModel notificationModel =
          NotificationModel.fromDocument(document);
      return Container(
        padding: EdgeInsets.symmetric(vertical: 5.0), //.all(4.0),
        child: NotificationsListTile(notificationModel: notificationModel),
      );
    } else {
      return SizedBox.shrink();
    }
  }
}
