/// MuixApp
///
/// Prototipus d’aplicació mòbil Android/iOS per a la gestió d’assajos, actuacions i catàlegs de figures de colles muixerangueres.
///
/// Grau en Enginyeria Informàtica.
/// Treball Final de Grau. Àrea Desenvolupament multiplataforma d'aplicacions mòbils.
/// Universitat Oberta de Catalunya.
/// Curs 2021-22.
///
/// Estudiant: Hèctor Rubio i Marín (hrubio@uoc.edu)
///
/// MIT License
/// Copyright (c) 2021 Hèctor Rubio i Marín

import 'package:flutter/material.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:material_design_icons_flutter/material_design_icons_flutter.dart';
import 'package:muixapp/src/models/profile_model.dart';
import 'package:muixapp/src/ui/widgets/authentication/no_access_no_logged_in_user.dart';
import 'package:muixapp/src/ui/widgets/people/people_list_tile.dart';
import 'package:provider/provider.dart';
import 'package:muixapp/src/providers/providers.dart';

class PeopleList extends StatefulWidget {
  PeopleList({Key? key}) : super(key: key);

  @override
  State createState() => PeopleListState();
}

class PeopleListState extends State<PeopleList> {
  PeopleListState({Key? key});

  TextEditingController searchController = TextEditingController();

  late ProfilesProvider profilesProvider;
  late AuthenticationProvider authenticationProvider;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    authenticationProvider = Provider.of<AuthenticationProvider>(context);
    profilesProvider = context.read<ProfilesProvider>();
  }

  @override
  Widget build(BuildContext context) {
    if (authenticationProvider.isLoggedIn())
      return Container(
        child: Column(children: [
          SizedBox(height: 10),
          Container(
            padding: EdgeInsets.symmetric(horizontal: 8.0), //.all(4.0),
            child: TextField(
              onChanged: (value) {
                setState(() {});
              },
              controller: searchController,
              decoration: InputDecoration(
                  hintText: 'Cerca',
                  prefixIcon: Icon(MdiIcons.magnify),
                  border: OutlineInputBorder(
                      borderRadius: BorderRadius.all(Radius.circular(5.0)))),
            ),
          ),
          SizedBox(height: 4),
          Flexible(
              fit: FlexFit.tight,
              child: Container(
                child: StreamBuilder<QuerySnapshot>(
                  stream: profilesProvider.getProfilesStream(),
                  builder: (context, AsyncSnapshot<QuerySnapshot> snapshot) {
                    if (snapshot.hasData) {
                      int counter = (snapshot.data?.docs.length ?? 0);
                      if (counter > 0) // return Text(counter.toString());
                        return ListView.builder(
                          padding: EdgeInsets.symmetric(horizontal: 8),
                          itemBuilder: (context, index) => buildItem(
                            context,
                            snapshot.data?.docs[index],
                            (searchController.text.isEmpty
                                ? ''
                                : searchController.text.toLowerCase()),
                          ),
                          itemCount: snapshot.data?.docs.length,
                        );
                    } else if (snapshot.hasError) {
                      return Text(snapshot.error.toString());
                    }
                    return Center(child: CircularProgressIndicator());
                  },
                ),
              ))
        ]),
      );
    else
      return NoAccessNoLoggedInUser();
  }

  Widget buildItem(
      BuildContext context, DocumentSnapshot? document, String searchFilter) {
    if (document != null) {
      ProfileModel profileModel = ProfileModel.fromDocument(document);
      if (profileModel.firstName.toLowerCase().contains(searchFilter) ||
          profileModel.lastName.toLowerCase().contains(searchFilter) ||
          profileModel.nickName.toLowerCase().contains(searchFilter))
        return Container(
          padding: EdgeInsets.symmetric(vertical: 5.0), //.all(4.0),
          child: PeopleListTile(profileModel, context),
        );
      else
        return SizedBox.shrink();
    } else {
      return SizedBox.shrink();
    }
  }
}
