/// MuixApp
///
/// Prototipus d’aplicació mòbil Android/iOS per a la gestió d’assajos, actuacions i catàlegs de figures de colles muixerangueres.
///
/// Grau en Enginyeria Informàtica.
/// Treball Final de Grau. Àrea Desenvolupament multiplataforma d'aplicacions mòbils.
/// Universitat Oberta de Catalunya.
/// Curs 2021-22.
///
/// Estudiant: Hèctor Rubio i Marín (hrubio@uoc.edu)
///
/// MIT License
/// Copyright (c) 2021 Hèctor Rubio i Marín

import 'package:flutter/material.dart';
import 'package:muixapp/src/models/profile_model.dart';
import 'package:muixapp/src/ui/resources/app_styles.dart';
import 'package:muixapp/src/ui/resources/resources.dart';
import 'package:muixapp/src/ui/widgets/authentication/no_access_no_logged_in_user.dart';
import 'package:muixapp/src/ui/widgets/common/app_page_bar.dart';
import 'package:provider/provider.dart';
import 'package:muixapp/src/ui/resources/app_colours.dart';
import 'package:muixapp/src/ui/resources/app_texts.dart';
import 'package:muixapp/src/providers/providers.dart';

class PeopleProfile extends StatefulWidget {
  final ProfileModel profileModel;
  const PeopleProfile({required this.profileModel, Key? key}) : super(key: key);

  @override
  _PeopleProfileState createState() => _PeopleProfileState(this.profileModel);
}

class _PeopleProfileState extends State<PeopleProfile> {
  final ProfileModel profileModel;

  _PeopleProfileState(
      this.profileModel);

  late PreferencesProvider preferencesProvider;
  late AuthenticationProvider authenticationProvider;
  late ProfilesProvider profilesProvider;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    preferencesProvider = Provider.of<PreferencesProvider>(context);
    authenticationProvider = Provider.of<AuthenticationProvider>(context);
    profilesProvider = Provider.of<ProfilesProvider>(context);
  }

  @override
  Widget build(BuildContext context) {

    if (authenticationProvider.isLoggedIn())
      return Scaffold(
        appBar: AppPageBar(
          leadingText: AppTexts.teamNavigation,
          barActions: [],
        ),
        body:
        Container(
          margin: EdgeInsets.fromLTRB(10, 20, 10, 10),
          padding: EdgeInsets.all(10),
          decoration: BoxDecoration(
            color: AppColours.lightGray,
          ),
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Row(
                children: [
                  (profileModel.isManager
                      ? AppIcons.profileManager
                      : AppIcons.profileStandardMember),
                  Column(
                    mainAxisAlignment: MainAxisAlignment.start,
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Text(
                          (profileModel.isManager || profileModel.isTrainer
                              ? (profileModel.isTrainer
                              ? AppTexts.profileTrainerRole
                              : AppTexts.profileManagerRole)
                              : AppTexts.profileStandardRole),
                          style: (profileModel.isManager ||
                              profileModel.isTrainer
                              ? AppStyles.profileManagerColouredTextStyle
                              : AppStyles
                              .profileStandardMemberColouredTextStyle)),
                      Text(
                        (profileModel.firstName.isNotEmpty
                            ? profileModel.firstName
                            : ''),
                        style: AppStyles.standardTextStyle,
                      ),
                      Text(
                        (profileModel.lastName.isNotEmpty
                            ? profileModel.lastName.toUpperCase()
                            : ''),
                        style: AppStyles.standardTextStyle,
                      ),
                    ],
                  )
                ],
              ),
              SizedBox(height: 10),
              Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Text(
                    AppTexts.profileBio,
                    style: (profileModel.isManager
                        ? AppStyles.profileManagerColouredTextStyle
                        : AppStyles
                        .profileStandardMemberColouredTextStyle),
                  ),
                  Text(
                  (profileModel.bio.isNotEmpty
                  ?profileModel.bio:profileModel.firstName+' '+AppTexts.profileNoBioText),
                    style: AppStyles.standardTextStyle,
                  )
                ],
              ),
              SizedBox(height: 10),

              (authenticationProvider.isManager() && (profileModel.email.isNotEmpty ||
                  profileModel.mobilePhone.isNotEmpty)
                  ? Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Text(
                    AppTexts.profileContactData,
                    style: (profileModel.isManager
                        ? AppStyles.profileManagerColouredTextStyle
                        : AppStyles
                        .profileStandardMemberColouredTextStyle),
                  ),
                  if (profileModel.email.isNotEmpty)
                    Text(
                      AppTexts.profileEmail + profileModel.email,
                      style: AppStyles.standardTextStyle,
                    ),
                  if (profileModel.mobilePhone.isNotEmpty)
                    Text(
                      AppTexts.profileMobilePhone +
                          profileModel.mobilePhone,
                      style: AppStyles.standardTextStyle,
                    ),
                ],
              )
                  : Text('')),
            ],
            //color: AppColours.white,
          ),
        ),
      );
    else
      return NoAccessNoLoggedInUser();
  }

}
