/// MuixApp
///
/// Prototipus d’aplicació mòbil Android/iOS per a la gestió d’assajos, actuacions i catàlegs de figures de colles muixerangueres.
///
/// Grau en Enginyeria Informàtica.
/// Treball Final de Grau. Àrea Desenvolupament multiplataforma d'aplicacions mòbils.
/// Universitat Oberta de Catalunya.
/// Curs 2021-22.
///
/// Estudiant: Hèctor Rubio i Marín (hrubio@uoc.edu)
///
/// MIT License
/// Copyright (c) 2021 Hèctor Rubio i Marín

import 'package:flutter/material.dart';
import 'package:muixapp/src/models/activity_model.dart';
import 'package:muixapp/src/ui/pages/activities/activity.dart';
import 'package:muixapp/src/ui/resources/resources.dart';
import 'package:muixapp/src/ui/widgets/activities/activity_type_icon.dart';
import 'package:muixapp/src/ui/widgets/activities/activity_type_text.dart';

class ActivitiesListTile extends Container {
  ActivitiesListTile(ActivityModel activity, BuildContext context)
      : super(
          height: 90,
          color: AppColours.lightGray,
          padding: EdgeInsets.symmetric(vertical: 5.0),
          child: Column(
            children: [
              Row(
                children: [
                  // Left column - Icon
                  ActivityTypeIcon(activity.type),
                  // Central column - Activity info
                  Flexible(
                    fit: FlexFit.tight,
                    child: Container(
                      margin: EdgeInsets.symmetric(vertical: 5.0),
                      height: 70,
                      child: Column(
                        mainAxisAlignment: MainAxisAlignment.start,
                        children: [
                          Row(
                            mainAxisAlignment: MainAxisAlignment.spaceBetween,
                            children: [
                              ActivityTypeText(activity.type),
                              Text(
                                activity.dateTime + " h",
                                style: TextStyle(
                                  color: AppColours.gray,
                                ),
                              ),
                            ],
                          ),
                          SizedBox(height: 8),
                          Row(
                            crossAxisAlignment: CrossAxisAlignment.start,
                            children: [
                              Expanded(
                                child: Column(
                                  crossAxisAlignment: CrossAxisAlignment.start,
                                  children: [
                                    Text(
                                      activity.title,
                                      overflow: TextOverflow.ellipsis,
                                      maxLines: 2,
                                      softWrap: true,
                                    ),
                                  ],
                                ),
                              ),
                            ],
                          ),
                        ],
                      ),
                    ),
                  ),
                  // Right column - actions
                  IconButton(
                    icon: AppIcons.goToPageIcon,
                    onPressed: () {
                      Navigator.push(
                        context,
                        MaterialPageRoute(builder: (context) {
                          return Activity(
                            activity: activity,
                            appMainBarBackText:
                            AppTexts.activitiesPageTitle,
                          );
                        }),
                      );
                    },
                  ),
                ],
              ),
            ],
          ),
        );
}

