/// MuixApp
///
/// Prototipus d’aplicació mòbil Android/iOS per a la gestió d’assajos, actuacions i catàlegs de figures de colles muixerangueres.
///
/// Grau en Enginyeria Informàtica.
/// Treball Final de Grau. Àrea Desenvolupament multiplataforma d'aplicacions mòbils.
/// Universitat Oberta de Catalunya.
/// Curs 2021-22.
///
/// Estudiant: Hèctor Rubio i Marín (hrubio@uoc.edu)
///
/// MIT License
/// Copyright (c) 2021 Hèctor Rubio i Marín

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:muixapp/src/models/activity_figure_model.dart';
import 'package:muixapp/src/models/attendee_model.dart';
import 'package:muixapp/src/providers/providers.dart';
import 'package:muixapp/src/ui/pages/activities/attendance_emotion.dart';
import 'package:muixapp/src/ui/resources/app_constants.dart';
import 'package:muixapp/src/ui/resources/resources.dart';
import 'package:provider/provider.dart';

class UserAttendance extends StatefulWidget {
  final String activityId;
  final String userId;
  final String nickName;

  const UserAttendance({required this.activityId, required this.userId, required this.nickName, Key? key}) : super(key: key);

  @override
  _UserAttendanceState createState() => _UserAttendanceState(activityId: this.activityId, userId: this.userId, nickName: this.nickName);
}

class _UserAttendanceState extends State<UserAttendance> {
  _UserAttendanceState({required this.activityId, required this.userId, required this.nickName
  });

  String activityId;
  String userId;
  String nickName;

  String? attendeeId;

  int attendanceValue = AppConstants.unansweredAttendingActivity;

  AttendeeModel? attendeeModel;

  late AttendeesProvider attendeesProvider;
  late ActivityFiguresProvider activityFiguresProvider;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    attendeesProvider = Provider.of<AttendeesProvider>(context);
    activityFiguresProvider = Provider.of<ActivityFiguresProvider>(context);
  }

  @override
  Widget build(BuildContext context) {

    return StreamBuilder<QuerySnapshot>(
      //stream: useAttendanceStream,
      stream: attendeesProvider.getAttendeeStreamByActivityAndUser(activityId, userId),
      builder: (BuildContext context, AsyncSnapshot<QuerySnapshot> snapshot) {
        if (snapshot.hasData) {

          if (snapshot.data!= null && snapshot.data!.size>0){
            attendeeModel =AttendeeModel.fromDocument(snapshot.data!.docs[0]);
            attendanceValue=attendeeModel!.value;
          }
          else {
            attendanceValue=AppConstants.unansweredAttendingActivity;
          }

          return Container(
            child:Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: [
                ButtonTheme(
                  minWidth: 90.0,
                  child: MaterialButton(
                    height: 30,
                    color: (attendanceValue==AppConstants.notAttendingActivity?AppColours.blue:AppColours.midGray),
                    onPressed: () {
                      notAttending(activityId, userId, nickName);
                      Navigator.push(
                        context,
                        MaterialPageRoute(builder: (context) => AttendanceEmotion(emotion: AppConstants.notAttendingActivity)),
                      ).then((value) => setState((){}));
                    },
                    child: Text(
                      AppTexts.notAttendingActivity,
                      style: (attendanceValue==AppConstants.notAttendingActivity?
                      AppStyles.userAttendanceSelectedButtonTextStyle:
                      AppStyles.userAttendanceStandardButtonTextStyle),
                    ),
                  ),
                ),
                ButtonTheme(
                  minWidth: 90.0,
                  child: MaterialButton(
                    height: 30,
                    color: (attendanceValue==AppConstants.maybeAttendingActivity?AppColours.blue:AppColours.midGray),
                    onPressed: () {
                      maybeAttending(activityId, userId, nickName);
                      Navigator.push(
                        context,
                        MaterialPageRoute(builder: (context) => AttendanceEmotion(emotion: AppConstants.maybeAttendingActivity)),
                      ).then((value) => setState((){}));
                    },
                    child: Text(
                      AppTexts.maybeAttendingActivity,
                      style: (attendanceValue==AppConstants.maybeAttendingActivity?
                      AppStyles.userAttendanceSelectedButtonTextStyle:
                      AppStyles.userAttendanceStandardButtonTextStyle),
                    ),
                  ),
                ),
                ButtonTheme(
                  minWidth: 90.0,
                  child: MaterialButton(
                    height: 30,
                    color: (attendanceValue==AppConstants.attendingActivity?AppColours.blue:AppColours.midGray),
                    onPressed: () {
                      attendeesProvider.attending(activityId, userId, nickName);
                      Navigator.push(
                        context,
                        MaterialPageRoute(builder: (context) => AttendanceEmotion(emotion: AppConstants.attendingActivity)),
                      ).then((value) => setState((){}));
                    },
                    child: Text(
                      AppTexts.attendingActivity,
                      style: (attendanceValue==AppConstants.attendingActivity?
                      AppStyles.userAttendanceSelectedButtonTextStyle:
                      AppStyles.userAttendanceStandardButtonTextStyle),
                    ),
                  ),
                ),
              ],
            ),
          );

        } else if (snapshot.hasError) {
          return Container(
            height: 50,
            padding: EdgeInsets.symmetric(vertical: 5.0),
            child: Column(
              children: [
                Text(AppTexts.futureError),
                CircularProgressIndicator(),
              ],
            ),
          );
        }
        return Container(
          height: 50,
          padding: EdgeInsets.symmetric(vertical: 5.0),
          child: Column(
            children: [
              CircularProgressIndicator(),
            ],
          ),
        );
      },
    );

  }

  void notAttending(String activityId, String userId, String nickName) {
    attendeesProvider.notAttending(activityId, userId, nickName);
    removeAttendeeFromActivityFigures(activityId,userId);
  }

  void maybeAttending(String activityId, String userId, String nickName) {
    attendeesProvider.maybeAttending(activityId, userId, nickName);
    removeAttendeeFromActivityFigures(activityId,userId);
  }

  void removeAttendeeFromActivityFigures(String activityId, String userId) {
    Future<QuerySnapshot> activityFiguresSnapshots = activityFiguresProvider.getActivityFiguresByActivity(activityId);
    Future<QuerySnapshot> attendeeSnapshots = attendeesProvider.getAttendeeByActivityAndUser(activityId, userId);

    attendeeSnapshots.then((QuerySnapshot attendeeSnapshot) => {
      attendeeSnapshot.docs.forEach((element) {

        activityFiguresSnapshots.then((QuerySnapshot snapshot) => {
          snapshot.docs.forEach((doc) {
            activityFiguresProvider.removeAttendee(ActivityFigureModel.fromDocument(doc), AttendeeModel.fromDocument(element));

          })
        });

      })
    });
  }
}

