/// MuixApp
///
/// Prototipus d’aplicació mòbil Android/iOS per a la gestió d’assajos, actuacions i catàlegs de figures de colles muixerangueres.
///
/// Grau en Enginyeria Informàtica.
/// Treball Final de Grau. Àrea Desenvolupament multiplataforma d'aplicacions mòbils.
/// Universitat Oberta de Catalunya.
/// Curs 2021-22.
///
/// Estudiant: Hèctor Rubio i Marín (hrubio@uoc.edu)
///
/// MIT License
/// Copyright (c) 2021 Hèctor Rubio i Marín

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';
import 'package:muixapp/src/models/activity_model.dart';
import 'package:muixapp/src/models/notification_model.dart';
import 'package:muixapp/src/providers/providers.dart';
import 'package:muixapp/src/ui/pages/activities/activity.dart';
import 'package:muixapp/src/ui/resources/resources.dart';
import 'package:muixapp/src/ui/widgets/activities/activity_type_icon.dart';
import 'package:muixapp/src/ui/widgets/activities/activity_type_text.dart';
import 'package:muixapp/src/ui/widgets/activities/user_attendance.dart';
import 'package:provider/provider.dart';

class NotificationsListTile extends StatefulWidget {
  const NotificationsListTile({required this.notificationModel, Key? key}) : super(key: key);
  final NotificationModel notificationModel;

  @override
  _NotificationsListTileState createState() => _NotificationsListTileState(notificationModel: this.notificationModel);
}

class _NotificationsListTileState extends State<NotificationsListTile> {

  _NotificationsListTileState({required this.notificationModel});
  final NotificationModel notificationModel;

  late PreferencesProvider preferencesProvider;
  late ActivitiesProvider activitiesProvider;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    preferencesProvider = Provider.of<PreferencesProvider>(context);
    activitiesProvider = Provider.of<ActivitiesProvider>(context);
  }

  @override
  Widget build(BuildContext context) {

    if (notificationModel.callToAction) {

      // notification with a call to action

      ActivityModel activity;

      return FutureBuilder(
        future: activitiesProvider.getActivity(notificationModel.relatedId),
        builder: (BuildContext context, AsyncSnapshot<DocumentSnapshot> snapshot){
          if (snapshot.hasData){
            activity = ActivityModel.fromDocument(snapshot.data!);
            return Container(
              height: 150,
              color: AppColours.lightGray,
              padding: EdgeInsets.symmetric(vertical: 5.0),
              child: Column(
                children: [
                  Container(
                      child: Column (
                          children:[
                            // Main data row

                            Row(
                              children: [
                                // Left column - Icon
                                ActivityTypeIcon(activity.type),
                                // Central column - Activity info
                                Flexible(
                                  fit: FlexFit.tight,
                                  child: Container(
                                    margin: EdgeInsets.symmetric(vertical: 5.0),
                                    height: 80,
                                    child: Column(
                                      mainAxisAlignment: MainAxisAlignment.start,
                                      children: [
                                        SizedBox(
                                          height: 4.0,
                                        ),
                                        Row(
                                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                                          children: [
                                            ActivityTypeText(activity.type),
                                            Text(
                                              activity.dateTime + " h",
                                              style: AppStyles.activityListTileDateTimeStyle,
                                            ),
                                          ],
                                        ),
                                        SizedBox(height: 8),
                                        Row(
                                          crossAxisAlignment: CrossAxisAlignment.start,
                                          children: [
                                            Expanded(
                                              child: Column(
                                                crossAxisAlignment: CrossAxisAlignment.start,
                                                children: [
                                                  Text(
                                                    activity.title,
                                                    overflow: TextOverflow.ellipsis,
                                                    maxLines: 3,
                                                    softWrap: true,
                                                  ),
                                                ],
                                              ),
                                            ),
                                          ],
                                        ),
                                      ],
                                    ),
                                  ),
                                ),
                                IconButton(
                                  icon: AppIcons.goToPageIcon,
                                  onPressed: () {
                                    Navigator.push(
                                      context,
                                      MaterialPageRoute(builder: (context) {
                                        return Activity(
                                          activity: activity,
                                          appMainBarBackText: AppTexts.notificationsPageTitle,
                                        );
                                      }),
                                    );
                                  },
                                ),
                              ],
                            ),

                            // User attendance row
                            Container(
                              margin: EdgeInsets.symmetric(horizontal: 10.0),
                              child: UserAttendance(
                                activityId:activity.id,
                                userId: preferencesProvider.getKeyStringValue('uid')!,
                                nickName: preferencesProvider.getKeyStringValue('usernickname')!,
                              ),
                            ),
                          ]
                      )// RSVP row
                  )
                ],
              ),
            );

          } else if (snapshot.hasError){
            return Container(
              height: 90,
              color: AppColours.lightGray,
              padding: EdgeInsets.symmetric(vertical: 5.0),
              child: Column(
                children: [
                  Text(AppTexts.futureError),
                  CircularProgressIndicator(),
                ],
              ),
            );
          }
          return Container(
            height: 90,
            color: AppColours.lightGray,
            padding: EdgeInsets.symmetric(vertical: 5.0),
            child: Column(
              children: [
                Text(AppTexts.futureLoading),
                CircularProgressIndicator(),
              ],
            ),
          );
        },
      );

    } else {

        // informative notification

        return Container(
            height: 100,
            color: AppColours.lightGray,
            padding: EdgeInsets.symmetric(vertical: 5.0),
            child: Column(
              children: [
                Row(
                  children: [
                    // Left column - Icon
                    AppIcons.informativeNotification,
                    // Central column - Activity info
                    Flexible(
                      fit: FlexFit.tight,
                      child: Container(
                        margin: EdgeInsets.symmetric(vertical: 5.0),
                        height: 80,
                        child: Column(
                          mainAxisAlignment: MainAxisAlignment.start,
                          children: [
                            SizedBox(
                              height: 4.0,
                            ),
                            Row(
                              mainAxisAlignment: MainAxisAlignment.spaceBetween,
                              children: [
                                Text(AppTexts.informativeNotificationType.toUpperCase(),style: AppStyles.notificationListTileTypeInformativeStyle),
                                SizedBox(width: 25,),
                                Text(
                                  notificationModel.dateTime + " h",
                                  style: AppStyles.activityListTileDateTimeStyle,
                                ),
                                SizedBox(width: 5,)
                              ],
                            ),
                            SizedBox(height: 8),
                            Row(
                              crossAxisAlignment: CrossAxisAlignment.start,
                              children: [
                                Expanded(
                                  child: Column(
                                    crossAxisAlignment: CrossAxisAlignment.start,
                                    children: [
                                      Text(
                                        notificationModel.message,
                                        overflow: TextOverflow.ellipsis,
                                        maxLines: 3,
                                        softWrap: true,
                                      ),
                                    ],
                                  ),
                                ),
                              ],
                            ),
                          ],
                        ),
                      ),
                    ),
                    SizedBox(width: 20),
                  ],
                ),
              ],
            ),
          );
      }
  }
}


